#!/usr/bin/env python3
#
# Copyright 2008,2009,2011,2012,2015 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
"""
Provide a base flow graph for USRP signal generators.
"""

import math
import sys
try:
    from uhd_app import UHDApp
except ImportError:
    from gnuradio.uhd.uhd_app import UHDApp
from gnuradio import gr, uhd, eng_notation, eng_arg
from gnuradio import analog
from gnuradio import blocks
from gnuradio.gr.pubsub import pubsub

# PyLint can't reliably detect C++ exports in modules, so let's disable that
# pylint: disable=no-member

DESC_KEY = 'desc'
SAMP_RATE_KEY = 'samp_rate'
LINK_RATE_KEY = 'link_rate'
GAIN_KEY = 'gain'
TX_FREQ_KEY = 'tx_freq'
DSP_FREQ_KEY = 'dsp_freq'
RF_FREQ_KEY = 'rf_freq'
AMPLITUDE_KEY = 'amplitude'
AMPL_RANGE_KEY = 'ampl_range'
WAVEFORM_FREQ_KEY = 'waveform_freq'
WAVEFORM_OFFSET_KEY = 'waveform_offset'
WAVEFORM2_FREQ_KEY = 'waveform2_freq'
FREQ_RANGE_KEY = 'freq_range'
TYPE_KEY = 'type'

WAVEFORMS = {
    analog.GR_CONST_WAVE: "Constant",
    analog.GR_SIN_WAVE: "Complex Sinusoid",
    analog.GR_GAUSSIAN: "Gaussian Noise",
    analog.GR_UNIFORM: "Uniform Noise",
    "2tone": "Two Tone",
    "sweep": "Sweep",
}


class USRPSiggen(gr.top_block, pubsub, UHDApp):
    """
    GUI-unaware GNU Radio flowgraph.  This may be used either with command
    line applications or GUI applications.
    """
    MIN_AMP_POWER_MODE = .001

    def __init__(self, args):
        gr.top_block.__init__(self)
        pubsub.__init__(self)
        if not 0.0 <= args.amplitude <= 1.0:
            raise ValueError(
                "Invalid value for amplitude: {}. Must be in [0.0, 1.0]"
                .format(args.amplitude))
        # If the power argument is given, we need to turn that into a power
        # *reference* level. This is a bit of a hack because we're assuming
        # knowledge of UHDApp (i.e. we're leaking abstractions). But it's simple
        # and harmless enough.
        if args.power:
            if args.amplitude < self.MIN_AMP_POWER_MODE:
                raise RuntimeError(
                    "[ERROR] Invalid amplitude: In power mode, amplitude must be "
                    "larger than {}!".format(self.MIN_AMP_POWER_MODE))
            args.power -= 20 * math.log10(args.amplitude)
        UHDApp.__init__(self, args=args, prefix="UHD-SIGGEN")
        self.extra_sink = None

        # Allocate some attributes
        self._src1 = None
        self._src2 = None
        self._src = None

        # Initialize device:
        self.setup_usrp(
            ctor=uhd.usrp_sink,
            args=args,
        )
        print("[UHD-SIGGEN] UHD Signal Generator")
        print(
            "[UHD-SIGGEN] UHD Version: {ver}".format(ver=uhd.get_version_string()))
        print("[UHD-SIGGEN] Using USRP configuration:")
        print(self.get_usrp_info_string(tx_or_rx="tx"))
        self.usrp_description = self.get_usrp_info_string(
            tx_or_rx="tx", compact=True)

        # Set subscribers and publishers:
        self.publish(SAMP_RATE_KEY, lambda: self.usrp.get_samp_rate())
        self.publish(DESC_KEY, lambda: self.usrp_description)
        self.publish(FREQ_RANGE_KEY,
                     lambda: self.usrp.get_freq_range(self.channels[0]))
        self.publish(GAIN_KEY, lambda: self.get_gain_or_power())

        self[SAMP_RATE_KEY] = args.samp_rate
        self[TX_FREQ_KEY] = args.freq
        self[AMPLITUDE_KEY] = args.amplitude
        self[WAVEFORM_FREQ_KEY] = args.waveform_freq
        self[WAVEFORM_OFFSET_KEY] = args.offset
        self[WAVEFORM2_FREQ_KEY] = args.waveform2_freq
        self[DSP_FREQ_KEY] = 0
        self[RF_FREQ_KEY] = 0

        # subscribe set methods
        self.subscribe(SAMP_RATE_KEY, self.set_samp_rate)
        self.subscribe(GAIN_KEY, self.set_gain_or_power)
        self.subscribe(TX_FREQ_KEY, self.set_freq)
        self.subscribe(AMPLITUDE_KEY, self.set_amplitude)
        self.subscribe(WAVEFORM_FREQ_KEY, self.set_waveform_freq)
        self.subscribe(WAVEFORM2_FREQ_KEY, self.set_waveform2_freq)
        self.subscribe(TYPE_KEY, self.set_waveform)
        self.subscribe(RF_FREQ_KEY, self.update_gain_range)

        # force update on pubsub keys
        for key in (SAMP_RATE_KEY, GAIN_KEY, TX_FREQ_KEY,
                    AMPLITUDE_KEY, WAVEFORM_FREQ_KEY,
                    WAVEFORM_OFFSET_KEY, WAVEFORM2_FREQ_KEY):
            self[key] = self[key]
        self[TYPE_KEY] = args.type  # set type last

    def set_samp_rate(self, samp_rate):
        """
        When sampling rate is updated, also update the signal sources.
        """
        self.vprint("Setting sampling rate to: {rate} Msps".format(
            rate=samp_rate / 1e6))
        self.usrp.set_samp_rate(samp_rate)
        samp_rate = self.usrp.get_samp_rate()
        if self[TYPE_KEY] in (analog.GR_SIN_WAVE, analog.GR_CONST_WAVE):
            self._src.set_sampling_freq(self[SAMP_RATE_KEY])
        elif self[TYPE_KEY] == "2tone":
            self._src1.set_sampling_freq(self[SAMP_RATE_KEY])
            self._src2.set_sampling_freq(self[SAMP_RATE_KEY])
        elif self[TYPE_KEY] == "sweep":
            self._src1.set_sampling_freq(self[SAMP_RATE_KEY])
            self._src2.set_sampling_freq(
                self[WAVEFORM_FREQ_KEY] * 2 * math.pi / self[SAMP_RATE_KEY])
        else:
            return True  # Waveform not yet set
        self.vprint("Set sample rate to: {rate} Msps".format(
            rate=samp_rate / 1e6))
        return True

    def set_waveform_freq(self, freq):
        " Change the frequency 1 of the generated waveform "
        if self[TYPE_KEY] == analog.GR_SIN_WAVE:
            self._src.set_frequency(freq)
        elif self[TYPE_KEY] == "2tone":
            self._src1.set_frequency(freq)
        elif self[TYPE_KEY] == 'sweep':
            # there is no set sensitivity, redo fg
            self[TYPE_KEY] = self[TYPE_KEY]
        return True

    def set_waveform2_freq(self, freq):
        """
        Change the frequency 2 of the generated waveform. This only
        applies to 2-tone and sweep.
        """
        if freq is None:
            self[WAVEFORM2_FREQ_KEY] = -self[WAVEFORM_FREQ_KEY]
            return
        if self[TYPE_KEY] == "2tone":
            self._src2.set_frequency(freq)
        elif self[TYPE_KEY] == "sweep":
            self._src1.set_frequency(freq)

    def set_waveform(self, waveform_type):
        """
        Select the generated waveform
        """
        self.vprint("Selecting waveform...")
        self.lock()
        self.disconnect_all()
        if waveform_type in (analog.GR_SIN_WAVE, analog.GR_CONST_WAVE):
            self._src = analog.sig_source_c(self[SAMP_RATE_KEY],       # Sample rate
                                            waveform_type,             # Waveform waveform_type
                                            # Waveform frequency
                                            self[WAVEFORM_FREQ_KEY],
                                            # Waveform amplitude
                                            self[AMPLITUDE_KEY],
                                            self[WAVEFORM_OFFSET_KEY])  # Waveform offset
        elif waveform_type in (analog.GR_GAUSSIAN, analog.GR_UNIFORM):
            self._src = analog.noise_source_c(
                waveform_type, self[AMPLITUDE_KEY])
        elif waveform_type == "2tone":
            self._src1 = analog.sig_source_c(self[SAMP_RATE_KEY],
                                             analog.GR_SIN_WAVE,
                                             self[WAVEFORM_FREQ_KEY],
                                             self[AMPLITUDE_KEY] / 2.0,
                                             0)
            if self[WAVEFORM2_FREQ_KEY] is None:
                self[WAVEFORM2_FREQ_KEY] = -self[WAVEFORM_FREQ_KEY]
            self._src2 = analog.sig_source_c(self[SAMP_RATE_KEY],
                                             analog.GR_SIN_WAVE,
                                             self[WAVEFORM2_FREQ_KEY],
                                             self[AMPLITUDE_KEY] / 2.0,
                                             0)
            self._src = blocks.add_cc()
            self.connect(self._src1, (self._src, 0))
            self.connect(self._src2, (self._src, 1))
        elif waveform_type == "sweep":
            # rf freq is center frequency
            # waveform_freq is total swept width
            # waveform2_freq is sweep rate
            # will sweep from (rf_freq-waveform_freq/2) to (rf_freq+waveform_freq/2)
            if self[WAVEFORM2_FREQ_KEY] is None:
                self[WAVEFORM2_FREQ_KEY] = 0.1
            self._src1 = analog.sig_source_f(self[SAMP_RATE_KEY],
                                             analog.GR_TRI_WAVE,
                                             self[WAVEFORM2_FREQ_KEY],
                                             1.0,
                                             -0.5)
            self._src2 = analog.frequency_modulator_fc(
                self[WAVEFORM_FREQ_KEY] * 2 * math.pi / self[SAMP_RATE_KEY])
            self._src = blocks.multiply_const_cc(self[AMPLITUDE_KEY])
            self.connect(self._src1, self._src2, self._src)
        else:
            raise RuntimeError("[UHD-SIGGEN] Unknown waveform waveform_type")
        for chan in range(len(self.channels)):
            self.connect(self._src, (self.usrp, chan))
        if self.extra_sink is not None:
            self.connect(self._src, self.extra_sink)
        self.unlock()
        self.vprint("Set baseband modulation to:", WAVEFORMS[waveform_type])
        n2s = eng_notation.num_to_str
        if waveform_type == analog.GR_SIN_WAVE:
            self.vprint("Modulation frequency: %sHz" %
                        (n2s(self[WAVEFORM_FREQ_KEY]),))
            self.vprint("Initial phase:", self[WAVEFORM_OFFSET_KEY])
        elif waveform_type == "2tone":
            self.vprint("Tone 1: %sHz" % (n2s(self[WAVEFORM_FREQ_KEY]),))
            self.vprint("Tone 2: %sHz" % (n2s(self[WAVEFORM2_FREQ_KEY]),))
        elif waveform_type == "sweep":
            self.vprint("Sweeping across {} Hz to {} Hz".format(
                n2s(-self[WAVEFORM_FREQ_KEY] / 2.0), n2s(self[WAVEFORM_FREQ_KEY] / 2.0)))
            self.vprint("Sweep rate: %sHz" % (n2s(self[WAVEFORM2_FREQ_KEY]),))
        self.vprint("TX amplitude:", self[AMPLITUDE_KEY])

    def set_amplitude(self, amplitude):
        """
        amplitude subscriber
        """
        if amplitude < 0.0 or amplitude > 1.0:
            self.vprint("Amplitude out of range:", amplitude)
            return False
        if self[TYPE_KEY] in (analog.GR_SIN_WAVE, analog.GR_CONST_WAVE, analog.GR_GAUSSIAN, analog.GR_UNIFORM):
            self._src.set_amplitude(amplitude)
        elif self[TYPE_KEY] == "2tone":
            self._src1.set_amplitude(amplitude / 2.0)
            self._src2.set_amplitude(amplitude / 2.0)
        elif self[TYPE_KEY] == "sweep":
            self._src.set_k(amplitude)
        else:
            return True  # Waveform not yet set
        self.vprint("Set amplitude to:", amplitude)
        self.update_gain_range()
        return True

    def get_gain_or_power(self):
        """
        Depending on gain type, return either a power level or the current gain
        """
        if self.gain_type == self.GAIN_TYPE_GAIN:
            return self.usrp.get_gain(self.channels[0])
        return self.usrp.get_power_reference(self.channels[0]) \
            + 20 * math.log10(self[AMPLITUDE_KEY])

    def set_gain_or_power(self, gain_or_power):
        """
        Call this if a gain or power value changed, but you're not sure which it
        is.

        If it's a power, we subtract the signal offset to generate a reference
        power.
        """
        if self.gain_type == self.GAIN_TYPE_POWER:
            self.set_power_reference(
                gain_or_power - 20 * math.log10(self[AMPLITUDE_KEY]))
        else:
            self.set_gain(gain_or_power)

    def update_gain_range(self):
        """
        Update self.gain_range.
        """
        if self.gain_type == self.GAIN_TYPE_POWER:
            if self[AMPLITUDE_KEY] < self.MIN_AMP_POWER_MODE:
                raise RuntimeError(
                    "[ERROR] Invalid amplitude: In power mode, amplitude must be "
                    "larger than {}!".format(self.MIN_AMP_POWER_MODE))
            power_range = self.usrp.get_power_range(self.channels[0])
            ampl_offset = 20 * math.log10(self[AMPLITUDE_KEY])
            self.gain_range = uhd.meta_range(
                math.floor(power_range.start() + ampl_offset),
                math.ceil(power_range.stop() + ampl_offset),
                power_range.step()
            )
            self.vprint("Updated power range to {:.2f} ... {:.2f} dBm.".format(
                self.gain_range.start(), self.gain_range.stop()))


def setup_argparser():
    """
    Create argument parser for signal generator.
    """
    parser = UHDApp.setup_argparser(
        description="USRP Signal Generator.",
        tx_or_rx="Tx",
    )
    group = parser.add_argument_group('Siggen Arguments')
    group.add_argument("-m", "--amplitude", type=eng_arg.eng_float, default=0.15,
                       help="Set output amplitude to AMPL (0.0-1.0). Note that "
                            "if --power is given, UHD will attempt to match the "
                            "output power regardless of the amplitude.",
                       metavar="AMPL")
    group.add_argument("-x", "--waveform-freq", type=eng_arg.eng_float, default=0.0,
                       help="Set baseband waveform frequency to FREQ")
    group.add_argument("-y", "--waveform2-freq", type=eng_arg.eng_float, default=0.0,
                       help="Set 2nd waveform frequency to FREQ")
    group.add_argument("--sine", dest="type", action="store_const", const=analog.GR_SIN_WAVE,
                       help="Generate a carrier modulated by a complex sine wave",
                       default=analog.GR_SIN_WAVE)
    group.add_argument("--const", dest="type", action="store_const", const=analog.GR_CONST_WAVE,
                       help="Generate a constant carrier")
    group.add_argument("--offset", type=eng_arg.eng_float, default=0,
                       help="Set waveform phase offset to OFFSET", metavar="OFFSET")
    group.add_argument("--gaussian", dest="type", action="store_const", const=analog.GR_GAUSSIAN,
                       help="Generate Gaussian random output")
    group.add_argument("--uniform", dest="type", action="store_const", const=analog.GR_UNIFORM,
                       help="Generate Uniform random output")
    group.add_argument("--2tone", dest="type", action="store_const", const="2tone",
                       help="Generate Two Tone signal for IMD testing")
    group.add_argument("--sweep", dest="type", action="store_const", const="sweep",
                       help="Generate a swept sine wave")
    return parser


def main():
    " Go, go, go! "
    if gr.enable_realtime_scheduling() != gr.RT_OK:
        print("Note: failed to enable realtime scheduling, continuing")
    # Grab command line args and create top block
    try:
        parser = setup_argparser()
        args = parser.parse_args()
        tb = USRPSiggen(args)
    except RuntimeError as ex:
        print(ex)
        sys.exit(1)
    tb.start()
    input('[UHD-SIGGEN] Press Enter to quit:\n')
    tb.stop()
    tb.wait()


if __name__ == "__main__":
    main()



#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

# The presence of this file turns this directory into a Python package

'''
Blocks and utilities for digital TV module.
'''

import os

try:
    from .dtv_python import *
except ImportError:
    dirname, filename = os.path.split(os.path.abspath(__file__))
    __path__.append(os.path.join(dirname, "bindings"))
    from .dtv_python import *

# Import pure python code here
from .atsc_rx import *



#!/usr/bin/env python
#
# Copyright 2007,2008,2010,2011 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

import numpy

try:
    from pylab import Button, connect, draw, figure, figtext, get_current_fig_manager, mlab, show, rcParams, ceil
except ImportError:
    print("Please install Matplotlib to run this script (http://matplotlib.sourceforge.net/)")
    raise SystemExit(1)

from argparse import ArgumentParser
from gnuradio.eng_arg import eng_float, intx
from gnuradio.plot_data import datatype_lookup


class plot_psd_base(object):
    def __init__(self, datatype, filename, options):
        self.hfile = open(filename, "r")
        self.block_length = options.block
        self.start = options.start
        self.sample_rate = options.sample_rate
        self.psdfftsize = options.psd_size
        self.specfftsize = options.spec_size

        self.dospec = options.enable_spec  # if we want to plot the spectrogram

        self.datatype = datatype
        if self.datatype is None:
            self.datatype = datatype_lookup[options.data_type]
        # number of bytes per sample in file
        self.sizeof_data = self.datatype().nbytes

        self.axis_font_size = 16
        self.label_font_size = 18
        self.title_font_size = 20
        self.text_size = 22

        # Setup PLOT
        self.fig = figure(1, figsize=(16, 12), facecolor='w')
        rcParams['xtick.labelsize'] = self.axis_font_size
        rcParams['ytick.labelsize'] = self.axis_font_size

        self.text_file = figtext(0.10, 0.95, ("File: %s" % filename),
                                 weight="heavy", size=self.text_size)
        self.text_file_pos = figtext(0.10, 0.92, "File Position: ",
                                     weight="heavy", size=self.text_size)
        self.text_block = figtext(0.35, 0.92, ("Block Size: %d" % self.block_length),
                                  weight="heavy", size=self.text_size)
        self.text_sr = figtext(0.60, 0.915, ("Sample Rate: %.2f" % self.sample_rate),
                               weight="heavy", size=self.text_size)
        self.make_plots()

        self.button_left_axes = self.fig.add_axes(
            [0.45, 0.01, 0.05, 0.05], frameon=True)
        self.button_left = Button(self.button_left_axes, "<")
        self.button_left_callback = self.button_left.on_clicked(
            self.button_left_click)

        self.button_right_axes = self.fig.add_axes(
            [0.50, 0.01, 0.05, 0.05], frameon=True)
        self.button_right = Button(self.button_right_axes, ">")
        self.button_right_callback = self.button_right.on_clicked(
            self.button_right_click)

        self.xlim = numpy.array(self.sp_iq.get_xlim())

        self.manager = get_current_fig_manager()
        connect('draw_event', self.zoom)
        connect('key_press_event', self.click)
        show()

    def get_data(self):
        self.position = self.hfile.tell() / self.sizeof_data
        self.text_file_pos.set_text("File Position: %d" % self.position)
        try:
            self.iq = numpy.fromfile(
                self.hfile, dtype=self.datatype, count=self.block_length)
        except MemoryError:
            print("End of File")
            return False
        else:
            # retesting length here as newer version of numpy does not throw a MemoryError, just
            # returns a zero-length array
            if(len(self.iq) > 0):
                tstep = 1.0 / self.sample_rate
                #self.time = numpy.array([tstep*(self.position + i) for i in range(len(self.iq))])
                self.time = numpy.array([tstep * (i)
                                        for i in range(len(self.iq))])

                self.iq_psd, self.freq = self.dopsd(self.iq)
                return True
            else:
                print("End of File")
                return False

    def dopsd(self, iq):
        ''' Need to do this here and plot later so we can do the fftshift '''
        overlap = self.psdfftsize / 4
        winfunc = numpy.blackman
        psd, freq = mlab.psd(iq, self.psdfftsize, self.sample_rate,
                             window=lambda d: d * winfunc(self.psdfftsize),
                             noverlap=overlap)
        psd = 10.0 * numpy.log10(abs(psd))
        return (psd, freq)

    def make_plots(self):
        # if specified on the command-line, set file pointer
        self.hfile.seek(self.sizeof_data * self.start, 1)

        iqdims = [[0.075, 0.2, 0.4, 0.6], [0.075, 0.55, 0.4, 0.3]]
        psddims = [[0.575, 0.2, 0.4, 0.6], [0.575, 0.55, 0.4, 0.3]]
        specdims = [0.2, 0.125, 0.6, 0.3]

        # Subplot for real and imaginary parts of signal
        self.sp_iq = self.fig.add_subplot(
            2, 2, 1, position=iqdims[self.dospec])
        self.sp_iq.set_title(
            ("I&Q"), fontsize=self.title_font_size, fontweight="bold")
        self.sp_iq.set_xlabel(
            "Time (s)", fontsize=self.label_font_size, fontweight="bold")
        self.sp_iq.set_ylabel(
            "Amplitude (V)", fontsize=self.label_font_size, fontweight="bold")

        # Subplot for PSD plot
        self.sp_psd = self.fig.add_subplot(
            2, 2, 2, position=psddims[self.dospec])
        self.sp_psd.set_title(
            ("PSD"), fontsize=self.title_font_size, fontweight="bold")
        self.sp_psd.set_xlabel(
            "Frequency (Hz)", fontsize=self.label_font_size, fontweight="bold")
        self.sp_psd.set_ylabel("Power Spectrum (dBm)",
                               fontsize=self.label_font_size, fontweight="bold")

        r = self.get_data()

        self.plot_iq = self.sp_iq.plot([], 'bo-')  # make plot for reals
        self.plot_iq += self.sp_iq.plot([], 'ro-')  # make plot for imags
        self.draw_time(self.time, self.iq)         # draw the plot

        self.plot_psd = self.sp_psd.plot([], 'b')  # make plot for PSD
        self.draw_psd(self.freq, self.iq_psd)      # draw the plot

        if self.dospec:
            # Subplot for spectrogram plot
            self.sp_spec = self.fig.add_subplot(2, 2, 3, position=specdims)
            self.sp_spec.set_title(
                ("Spectrogram"), fontsize=self.title_font_size, fontweight="bold")
            self.sp_spec.set_xlabel(
                "Time (s)", fontsize=self.label_font_size, fontweight="bold")
            self.sp_spec.set_ylabel(
                "Frequency (Hz)", fontsize=self.label_font_size, fontweight="bold")

            self.draw_spec(self.time, self.iq)

        draw()

    def draw_time(self, t, iq):
        reals = iq.real
        imags = iq.imag
        self.plot_iq[0].set_data([t, reals])
        self.plot_iq[1].set_data([t, imags])
        self.sp_iq.set_xlim(t.min(), t.max())
        self.sp_iq.set_ylim([1.5 * min([reals.min(), imags.min()]),
                             1.5 * max([reals.max(), imags.max()])])

    def draw_psd(self, f, p):
        self.plot_psd[0].set_data([f, p])
        self.sp_psd.set_ylim([p.min() - 10, p.max() + 10])
        self.sp_psd.set_xlim([f.min(), f.max()])

    def draw_spec(self, t, s):
        overlap = self.specfftsize / 4
        winfunc = numpy.blackman
        self.sp_spec.clear()
        self.sp_spec.specgram(s, self.specfftsize, self.sample_rate,
                              window=lambda d: d * winfunc(self.specfftsize),
                              noverlap=overlap, xextent=[t.min(), t.max()])

    def update_plots(self):
        self.draw_time(self.time, self.iq)
        self.draw_psd(self.freq, self.iq_psd)

        if self.dospec:
            self.draw_spec(self.time, self.iq)

        # so zoom doesn't get called
        self.xlim = numpy.array(self.sp_iq.get_xlim())

        draw()

    def zoom(self, event):
        newxlim = numpy.array(self.sp_iq.get_xlim())
        curxlim = numpy.array(self.xlim)
        if(newxlim[0] != curxlim[0] or newxlim[1] != curxlim[1]):
            #xmin = max(0, int(ceil(self.sample_rate*(newxlim[0] - self.position))))
            #xmax = min(int(ceil(self.sample_rate*(newxlim[1] - self.position))), len(self.iq))
            xmin = max(0, int(ceil(self.sample_rate * (newxlim[0]))))
            xmax = min(
                int(ceil(self.sample_rate * (newxlim[1]))), len(self.iq))

            iq = numpy.array(self.iq[xmin: xmax])
            time = numpy.array(self.time[xmin: xmax])

            iq_psd, freq = self.dopsd(iq)

            self.draw_psd(freq, iq_psd)
            self.xlim = numpy.array(self.sp_iq.get_xlim())

            draw()

    def click(self, event):
        forward_valid_keys = [" ", "down", "right"]
        backward_valid_keys = ["up", "left"]

        if(find(event.key, forward_valid_keys)):
            self.step_forward()

        elif(find(event.key, backward_valid_keys)):
            self.step_backward()

    def button_left_click(self, event):
        self.step_backward()

    def button_right_click(self, event):
        self.step_forward()

    def step_forward(self):
        r = self.get_data()
        if(r):
            self.update_plots()

    def step_backward(self):
        # Step back in file position
        if(self.hfile.tell() >= 2 * self.sizeof_data * self.block_length):
            self.hfile.seek(-2 * self.sizeof_data * self.block_length, 1)
        else:
            self.hfile.seek(-self.hfile.tell(), 1)
        r = self.get_data()
        if(r):
            self.update_plots()

    @staticmethod
    def setup_options():
        description = "Takes a GNU Radio binary file (with specified data type using --data-type) and displays the I&Q data versus time as well as the power spectral density (PSD) plot. The y-axis values are plotted assuming volts as the amplitude of the I&Q streams and converted into dBm in the frequency domain (the 1/N power adjustment out of the FFT is performed internally). The script plots a certain block of data at a time, specified on the command line as -B or --block. The start position in the file can be set by specifying -s or --start and defaults to 0 (the start of the file). By default, the system assumes a sample rate of 1, so in time, each sample is plotted versus the sample number. To set a true time and frequency axis, set the sample rate (-R or --sample-rate) to the sample rate used when capturing the samples. Finally, the size of the FFT to use for the PSD and spectrogram plots can be set independently with --psd-size and --spec-size, respectively. The spectrogram plot does not display by default and is turned on with -S or --enable-spec."

        parser = ArgumentParser(
            conflict_handler="resolve", description=description)
        parser.add_argument("-d", "--data-type", default="complex64",
                            choices=("complex64", "float32", "int32", "uint32", "int16",
                                     "uint16", "int8", "uint8"),
                            help="Specify the data type [default=%(default)r]")
        parser.add_argument("-B", "--block", type=int, default=8192,
                            help="Specify the block size [default=%(default)r]")
        parser.add_argument("-s", "--start", type=int, default=0,
                            help="Specify where to start in the file [default=%(default)r]")
        parser.add_argument("-R", "--sample-rate", type=eng_float, default=1.0,
                            help="Set the sampler rate of the data [default=%(default)r]")
        parser.add_argument("--psd-size", type=int, default=1024,
                            help="Set the size of the PSD FFT [default=%(default)r]")
        parser.add_argument("--spec-size", type=int, default=256,
                            help="Set the size of the spectrogram FFT [default=%(default)r]")
        parser.add_argument("-S", "--enable-spec", action="store_true",
                            help="Turn on plotting the spectrogram [default=%(default)r]")
        parser.add_argument("file", metavar="FILE",
                            help="Input file with samples")

        return parser


def find(item_in, list_search):
    try:
        return list_search.index(item_in) != None
    except ValueError:
        return False


def main():
    parser = plot_psd_base.setup_options()
    args = parser.parse_args()

    plot_psd_base(None, args.file, args)


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        pass



#
# Copyright 2019 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
""" constants file """

# Kernel Namespace
KERNEL = 'kernel'

# I/O Signature (Symbols and constants)
IO_SIGNATURE = 'io_signature::'
SIGNATURE_LIST = ['makev', 'make3', 'make2', 'make']
MAKE = 'make'
MAKE2 = 'make2'
MAKE3 = 'make3'
MAKEV = 'makev'


# message ports id
MESSAGE_INPUT = 'message_port_register_in'
MESSAGE_OUTPUT = 'message_port_register_out'

# Symbols and constants required for parsing
GR = 'gr-'
UTILS = 'utils'
OPEN_BRACKET = '('
CLOSE_BRACKET = ')'
STRIP_SYMBOLS = ' ,:)'
EXCLAMATION = '!'

# Blocktool special comments
BLOCKTOOL = '! BlockTool'
END_BLOCKTOOL = 'EndTool !'
INPUT_SIG = 'input_signature'
OUTPUT_SIG = 'output_signature'
INPUT_MIN = 'input_min_streams'
INPUT_MAX = 'input_max_streams'
OUTPUT_MIN = 'output_min_streams'
OUTPUT_MAX = 'output_max_streams'
INPUT_MAKE_SIZE = 'input_sizeof_stream_item'
INPUT_MAKEV_SIZE = 'input_sizeof_stream_items'
INPUT_MAKE_SIZE1 = 'input_sizeof_stream_item1'
INPUT_MAKE_SIZE2 = 'input_sizeof_stream_item2'
INPUT_MAKE_SIZE3 = 'input_sizeof_stream_item3'
OUTPUT_MAKE_SIZE = 'output_sizeof_stream_item'
OUTPUT_MAKEV_SIZE = 'output_sizeof_stream_items'
OUTPUT_MAKE_SIZE1 = 'output_sizeof_stream_item1'
OUTPUT_MAKE_SIZE2 = 'output_sizeof_stream_item2'
OUTPUT_MAKE_SIZE3 = 'output_sizeof_stream_item3'
INPUT_PORT = 'message_input'
OUTPUT_PORT = 'message_output'



#
# Autogenerated by Thrift Compiler (0.22.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec
from uuid import UUID

import sys
from .ttypes import *
DISPNULL = 0
DISPTIME = 1
DISPXY = 2
DISPPSD = 4
DISPSPEC = 8
DISPRAST = 16
DISPOPTCPLX = 256
DISPOPTLOG = 512
DISPOPTSTEM = 1024
DISPOPTSTRIP = 2048
DISPOPTSCATTER = 4096



#
# Copyright 2005,2006,2007 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
"""
Legacy utilities for packet modulation
"""

default_access_code_binary = b'\xAC\xDD\xA4\xE2\xF2\x8C\x20\xFC'
# This is the same as default_access_code_binary, but as a string of 1s and 0s
default_access_code = "1010110011011101101001001110001011110010100011000010000011111100"



#!/usr/bin/env python
#
# Copyright 2012 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import sys
import decimal
from gnuradio import gr, blocks
import pmt

'''
sr    Sample rate (samples/second)
time  Time as uint64(secs), double(fractional secs)
type  Type of data (see gr_file_types enum)
cplx  is complex? (True or False)
strt  Start of data (or size of header) in bytes
size  Size of data in bytes
'''

HEADER_LENGTH = blocks.METADATA_HEADER_SIZE

ftype_to_string = {blocks.GR_FILE_BYTE: "bytes",
                   blocks.GR_FILE_SHORT: "short",
                   blocks.GR_FILE_INT: "int",
                   blocks.GR_FILE_LONG: "long",
                   blocks.GR_FILE_LONG_LONG: "long long",
                   blocks.GR_FILE_FLOAT: "float",
                   blocks.GR_FILE_DOUBLE: "double"}

ftype_to_size = {blocks.GR_FILE_BYTE: gr.sizeof_char,
                 blocks.GR_FILE_SHORT: gr.sizeof_short,
                 blocks.GR_FILE_INT: gr.sizeof_int,
                 blocks.GR_FILE_LONG: gr.sizeof_int,
                 blocks.GR_FILE_LONG_LONG: 2 * gr.sizeof_int,
                 blocks.GR_FILE_FLOAT: gr.sizeof_float,
                 blocks.GR_FILE_DOUBLE: gr.sizeof_double}


def parse_header(p, VERBOSE=False):
    dump = pmt.PMT_NIL

    info = dict()

    if(pmt.is_dict(p) is False):
        sys.stderr.write(
            "Header is not a PMT dictionary: invalid or corrupt data file.\n")
        sys.exit(1)

    # GET FILE FORMAT VERSION NUMBER
    if(pmt.dict_has_key(p, pmt.string_to_symbol("version"))):
        r = pmt.dict_ref(p, pmt.string_to_symbol("version"), dump)
        version = pmt.to_long(r)
        if(VERBOSE):
            print("Version Number: {0}".format(version))
    else:
        sys.stderr.write(
            "Could not find key 'version': invalid or corrupt data file.\n")
        sys.exit(1)

    # EXTRACT SAMPLE RATE
    if(pmt.dict_has_key(p, pmt.string_to_symbol("rx_rate"))):
        r = pmt.dict_ref(p, pmt.string_to_symbol("rx_rate"), dump)
        samp_rate = pmt.to_double(r)
        info["rx_rate"] = samp_rate
        if(VERBOSE):
            print("Sample Rate: {0:.2f} sps".format(samp_rate))
    else:
        sys.stderr.write(
            "Could not find key 'sr': invalid or corrupt data file.\n")
        sys.exit(1)

    # EXTRACT TIME STAMP
    if(pmt.dict_has_key(p, pmt.string_to_symbol("rx_time"))):
        r = pmt.dict_ref(p, pmt.string_to_symbol("rx_time"), dump)
        secs = pmt.tuple_ref(r, 0)
        fracs = pmt.tuple_ref(r, 1)
        secs = pmt.to_uint64(secs)
        fracs = pmt.to_double(fracs)
        t = float(secs) + fracs
        info["rx_time_secs"] = secs
        info["rx_time_fracs"] = fracs
        info["rx_time"] = t
        if(VERBOSE):
            # We need are going to print with ~1e-16 resolution,
            # which is the precision we can expect in secs.
            # The default value of precision for decimal
            # is 28. The value of secs is not expected to be larger
            # than 1e12, so this precision is enough.
            s = decimal.Decimal(secs) + decimal.Decimal(fracs)
            print("Seconds: {0:.16f}".format(s))
    else:
        sys.stderr.write(
            "Could not find key 'time': invalid or corrupt data file.\n")
        sys.exit(1)

    # EXTRACT ITEM SIZE
    if(pmt.dict_has_key(p, pmt.string_to_symbol("size"))):
        r = pmt.dict_ref(p, pmt.string_to_symbol("size"), dump)
        dsize = pmt.to_long(r)
        info["size"] = dsize
        if(VERBOSE):
            print("Item size: {0}".format(dsize))
    else:
        sys.stderr.write(
            "Could not find key 'size': invalid or corrupt data file.\n")
        sys.exit(1)

    # EXTRACT DATA TYPE
    if(pmt.dict_has_key(p, pmt.string_to_symbol("type"))):
        r = pmt.dict_ref(p, pmt.string_to_symbol("type"), dump)
        dtype = pmt.to_long(r)
        stype = ftype_to_string[dtype]
        info["type"] = stype
        if(VERBOSE):
            print("Data Type: {0} ({1})".format(stype, dtype))
    else:
        sys.stderr.write(
            "Could not find key 'type': invalid or corrupt data file.\n")
        sys.exit(1)

    # EXTRACT COMPLEX
    if(pmt.dict_has_key(p, pmt.string_to_symbol("cplx"))):
        r = pmt.dict_ref(p, pmt.string_to_symbol("cplx"), dump)
        cplx = pmt.to_bool(r)
        info["cplx"] = cplx
        if(VERBOSE):
            print("Complex? {0}".format(cplx))
    else:
        sys.stderr.write(
            "Could not find key 'cplx': invalid or corrupt data file.\n")
        sys.exit(1)

    # EXTRACT WHERE CURRENT SEGMENT STARTS
    if(pmt.dict_has_key(p, pmt.string_to_symbol("strt"))):
        r = pmt.dict_ref(p, pmt.string_to_symbol("strt"), dump)
        seg_start = pmt.to_uint64(r)
        info["hdr_len"] = seg_start
        info["extra_len"] = seg_start - HEADER_LENGTH
        info["has_extra"] = info["extra_len"] > 0
        if(VERBOSE):
            print("Header Length: {0} bytes".format(info["hdr_len"]))
            print("Extra Length:  {0}".format((info["extra_len"])))
            print("Extra Header?  {0}".format(info["has_extra"]))
    else:
        sys.stderr.write(
            "Could not find key 'strt': invalid or corrupt data file.\n")
        sys.exit(1)

    # EXTRACT SIZE OF DATA
    if(pmt.dict_has_key(p, pmt.string_to_symbol("bytes"))):
        r = pmt.dict_ref(p, pmt.string_to_symbol("bytes"), dump)
        nbytes = pmt.to_uint64(r)

        nitems = nbytes / dsize
        info["nitems"] = nitems
        info["nbytes"] = nbytes

        if(VERBOSE):
            print("Size of Data: {0} bytes".format(nbytes))
            print("              {0} items".format(nitems))
    else:
        sys.stderr.write(
            "Could not find key 'size': invalid or corrupt data file.\n")
        sys.exit(1)

    return info

# IF THERE IS EXTRA DATA, PULL OUT THE DICTIONARY AND PARSE IT


def parse_extra_dict(p, info, VERBOSE=False):
    if(pmt.is_dict(p) is False):
        sys.stderr.write(
            "Extra header is not a PMT dictionary: invalid or corrupt data file.\n")
        sys.exit(1)

    items = pmt.dict_items(p)
    nitems = pmt.length(items)
    for i in range(nitems):
        item = pmt.nth(i, items)
        key = pmt.symbol_to_string(pmt.car(item))
        val = pmt.cdr(item)
        info[key] = val
        if(VERBOSE):
            print("{0}: {1}".format(key, val))

    return info



#
# Copyright 2005,2007,2012,2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import math

from gnuradio import gr
from gnuradio import filter

from . import analog_python as analog
from .fm_emph import fm_preemph


class wfm_tx(gr.hier_block2):
    def __init__(self, audio_rate, quad_rate, tau=75e-6, max_dev=75e3, fh=-1.0):
        """
        Wide Band FM Transmitter.

        Takes a single float input stream of audio samples in the range [-1,+1]
        and produces a single FM modulated complex baseband output.

        Args:
            audio_rate: sample rate of audio stream, >= 16k (integer)
            quad_rate: sample rate of output stream (integer)
            tau: preemphasis time constant (default 75e-6) (float)
            max_dev: maximum deviation in Hz (default 75e3) (float)
            fh: high frequency at which to flatten preemphasis; < 0 means default of 0.925*quad_rate/2.0 (float)

        quad_rate must be an integer multiple of audio_rate.
        """
        gr.hier_block2.__init__(self, "wfm_tx",
                                # Input signature
                                gr.io_signature(1, 1, gr.sizeof_float),
                                gr.io_signature(1, 1, gr.sizeof_gr_complex))  # Output signature

        # FIXME audio_rate and quad_rate ought to be exact rationals
        audio_rate = int(audio_rate)
        quad_rate = int(quad_rate)

        if quad_rate % audio_rate != 0:
            raise ValueError(
                "quad_rate is not an integer multiple of audio_rate")

        do_interp = audio_rate != quad_rate

        if do_interp:
            interp_factor = quad_rate // audio_rate
            interp_taps = filter.firdes.low_pass(interp_factor,  # gain
                                                 quad_rate,      # Fs
                                                 19000,          # cutoff_freq
                                                 4000)           # transition_width
            print("len(interp_taps) =", len(interp_taps))
            self.interpolator = filter.interp_fir_filter_fff(
                interp_factor, interp_taps)

        self.preemph = fm_preemph(quad_rate, tau=tau, fh=fh)

        k = 2 * math.pi * max_dev / quad_rate
        self.modulator = analog.frequency_modulator_fc(k)

        if do_interp:
            self.connect(self, self.interpolator,
                         self.preemph, self.modulator, self)
        else:
            self.connect(self, self.preemph, self.modulator, self)



#
# Autogenerated by Thrift Compiler (0.22.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec
from uuid import UUID

import sys

from thrift.transport import TTransport
all_structs = []


class BaseTypes(object):
    BOOL = 0
    BYTE = 1
    SHORT = 2
    INT = 3
    LONG = 4
    DOUBLE = 5
    STRING = 6
    COMPLEX = 7
    F32VECTOR = 8
    F64VECTOR = 9
    S64VECTOR = 10
    S32VECTOR = 11
    S16VECTOR = 12
    S8VECTOR = 13
    C32VECTOR = 14

    _VALUES_TO_NAMES = {
        0: "BOOL",
        1: "BYTE",
        2: "SHORT",
        3: "INT",
        4: "LONG",
        5: "DOUBLE",
        6: "STRING",
        7: "COMPLEX",
        8: "F32VECTOR",
        9: "F64VECTOR",
        10: "S64VECTOR",
        11: "S32VECTOR",
        12: "S16VECTOR",
        13: "S8VECTOR",
        14: "C32VECTOR",
    }

    _NAMES_TO_VALUES = {
        "BOOL": 0,
        "BYTE": 1,
        "SHORT": 2,
        "INT": 3,
        "LONG": 4,
        "DOUBLE": 5,
        "STRING": 6,
        "COMPLEX": 7,
        "F32VECTOR": 8,
        "F64VECTOR": 9,
        "S64VECTOR": 10,
        "S32VECTOR": 11,
        "S16VECTOR": 12,
        "S8VECTOR": 13,
        "C32VECTOR": 14,
    }


class KnobType(object):
    KNOBBOOL = 0
    KNOBCHAR = 1
    KNOBINT = 2
    KNOBDOUBLE = 3
    KNOBSTRING = 4
    KNOBLONG = 5
    KNOBVECBOOL = 6
    KNOBVECCHAR = 7
    KNOBVECINT = 8
    KNOBVECDOUBLE = 9
    KNOBVECSTRING = 10
    KNOBVECLONG = 11
    KNOBSHORT = 12

    _VALUES_TO_NAMES = {
        0: "KNOBBOOL",
        1: "KNOBCHAR",
        2: "KNOBINT",
        3: "KNOBDOUBLE",
        4: "KNOBSTRING",
        5: "KNOBLONG",
        6: "KNOBVECBOOL",
        7: "KNOBVECCHAR",
        8: "KNOBVECINT",
        9: "KNOBVECDOUBLE",
        10: "KNOBVECSTRING",
        11: "KNOBVECLONG",
        12: "KNOBSHORT",
    }

    _NAMES_TO_VALUES = {
        "KNOBBOOL": 0,
        "KNOBCHAR": 1,
        "KNOBINT": 2,
        "KNOBDOUBLE": 3,
        "KNOBSTRING": 4,
        "KNOBLONG": 5,
        "KNOBVECBOOL": 6,
        "KNOBVECCHAR": 7,
        "KNOBVECINT": 8,
        "KNOBVECDOUBLE": 9,
        "KNOBVECSTRING": 10,
        "KNOBVECLONG": 11,
        "KNOBSHORT": 12,
    }


class complex(object):
    """
    Attributes:
     - re
     - im

    """
    thrift_spec = None


    def __init__(self, re = None, im = None,):
        self.re = re
        self.im = im

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.DOUBLE:
                    self.re = iprot.readDouble()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.DOUBLE:
                    self.im = iprot.readDouble()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        self.validate()
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('complex')
        if self.re is not None:
            oprot.writeFieldBegin('re', TType.DOUBLE, 1)
            oprot.writeDouble(self.re)
            oprot.writeFieldEnd()
        if self.im is not None:
            oprot.writeFieldBegin('im', TType.DOUBLE, 2)
            oprot.writeDouble(self.im)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class KnobBase(object):
    """
    Attributes:
     - a_bool
     - a_byte
     - a_short
     - a_int
     - a_long
     - a_double
     - a_string
     - a_complex
     - a_f32vector
     - a_f64vector
     - a_s64vector
     - a_s32vector
     - a_s16vector
     - a_s8vector
     - a_c32vector

    """
    thrift_spec = None


    def __init__(self, a_bool = None, a_byte = None, a_short = None, a_int = None, a_long = None, a_double = None, a_string = None, a_complex = None, a_f32vector = None, a_f64vector = None, a_s64vector = None, a_s32vector = None, a_s16vector = None, a_s8vector = None, a_c32vector = None,):
        self.a_bool = a_bool
        self.a_byte = a_byte
        self.a_short = a_short
        self.a_int = a_int
        self.a_long = a_long
        self.a_double = a_double
        self.a_string = a_string
        self.a_complex = a_complex
        self.a_f32vector = a_f32vector
        self.a_f64vector = a_f64vector
        self.a_s64vector = a_s64vector
        self.a_s32vector = a_s32vector
        self.a_s16vector = a_s16vector
        self.a_s8vector = a_s8vector
        self.a_c32vector = a_c32vector

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.BOOL:
                    self.a_bool = iprot.readBool()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.BYTE:
                    self.a_byte = iprot.readByte()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.I16:
                    self.a_short = iprot.readI16()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.I32:
                    self.a_int = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.I64:
                    self.a_long = iprot.readI64()
                else:
                    iprot.skip(ftype)
            elif fid == 6:
                if ftype == TType.DOUBLE:
                    self.a_double = iprot.readDouble()
                else:
                    iprot.skip(ftype)
            elif fid == 7:
                if ftype == TType.STRING:
                    self.a_string = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 8:
                if ftype == TType.STRUCT:
                    self.a_complex = complex()
                    self.a_complex.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 9:
                if ftype == TType.LIST:
                    self.a_f32vector = []
                    (_etype3, _size0) = iprot.readListBegin()
                    for _i4 in range(_size0):
                        _elem5 = iprot.readDouble()
                        self.a_f32vector.append(_elem5)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 10:
                if ftype == TType.LIST:
                    self.a_f64vector = []
                    (_etype9, _size6) = iprot.readListBegin()
                    for _i10 in range(_size6):
                        _elem11 = iprot.readDouble()
                        self.a_f64vector.append(_elem11)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 11:
                if ftype == TType.LIST:
                    self.a_s64vector = []
                    (_etype15, _size12) = iprot.readListBegin()
                    for _i16 in range(_size12):
                        _elem17 = iprot.readI64()
                        self.a_s64vector.append(_elem17)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 12:
                if ftype == TType.LIST:
                    self.a_s32vector = []
                    (_etype21, _size18) = iprot.readListBegin()
                    for _i22 in range(_size18):
                        _elem23 = iprot.readI32()
                        self.a_s32vector.append(_elem23)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 13:
                if ftype == TType.LIST:
                    self.a_s16vector = []
                    (_etype27, _size24) = iprot.readListBegin()
                    for _i28 in range(_size24):
                        _elem29 = iprot.readI16()
                        self.a_s16vector.append(_elem29)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 14:
                if ftype == TType.STRING:
                    self.a_s8vector = iprot.readBinary()
                else:
                    iprot.skip(ftype)
            elif fid == 15:
                if ftype == TType.LIST:
                    self.a_c32vector = []
                    (_etype33, _size30) = iprot.readListBegin()
                    for _i34 in range(_size30):
                        _elem35 = complex()
                        _elem35.read(iprot)
                        self.a_c32vector.append(_elem35)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        self.validate()
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('KnobBase')
        if self.a_bool is not None:
            oprot.writeFieldBegin('a_bool', TType.BOOL, 1)
            oprot.writeBool(self.a_bool)
            oprot.writeFieldEnd()
        if self.a_byte is not None:
            oprot.writeFieldBegin('a_byte', TType.BYTE, 2)
            oprot.writeByte(self.a_byte)
            oprot.writeFieldEnd()
        if self.a_short is not None:
            oprot.writeFieldBegin('a_short', TType.I16, 3)
            oprot.writeI16(self.a_short)
            oprot.writeFieldEnd()
        if self.a_int is not None:
            oprot.writeFieldBegin('a_int', TType.I32, 4)
            oprot.writeI32(self.a_int)
            oprot.writeFieldEnd()
        if self.a_long is not None:
            oprot.writeFieldBegin('a_long', TType.I64, 5)
            oprot.writeI64(self.a_long)
            oprot.writeFieldEnd()
        if self.a_double is not None:
            oprot.writeFieldBegin('a_double', TType.DOUBLE, 6)
            oprot.writeDouble(self.a_double)
            oprot.writeFieldEnd()
        if self.a_string is not None:
            oprot.writeFieldBegin('a_string', TType.STRING, 7)
            oprot.writeString(self.a_string.encode('utf-8') if sys.version_info[0] == 2 else self.a_string)
            oprot.writeFieldEnd()
        if self.a_complex is not None:
            oprot.writeFieldBegin('a_complex', TType.STRUCT, 8)
            self.a_complex.write(oprot)
            oprot.writeFieldEnd()
        if self.a_f32vector is not None:
            oprot.writeFieldBegin('a_f32vector', TType.LIST, 9)
            oprot.writeListBegin(TType.DOUBLE, len(self.a_f32vector))
            for iter36 in self.a_f32vector:
                oprot.writeDouble(iter36)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.a_f64vector is not None:
            oprot.writeFieldBegin('a_f64vector', TType.LIST, 10)
            oprot.writeListBegin(TType.DOUBLE, len(self.a_f64vector))
            for iter37 in self.a_f64vector:
                oprot.writeDouble(iter37)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.a_s64vector is not None:
            oprot.writeFieldBegin('a_s64vector', TType.LIST, 11)
            oprot.writeListBegin(TType.I64, len(self.a_s64vector))
            for iter38 in self.a_s64vector:
                oprot.writeI64(iter38)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.a_s32vector is not None:
            oprot.writeFieldBegin('a_s32vector', TType.LIST, 12)
            oprot.writeListBegin(TType.I32, len(self.a_s32vector))
            for iter39 in self.a_s32vector:
                oprot.writeI32(iter39)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.a_s16vector is not None:
            oprot.writeFieldBegin('a_s16vector', TType.LIST, 13)
            oprot.writeListBegin(TType.I16, len(self.a_s16vector))
            for iter40 in self.a_s16vector:
                oprot.writeI16(iter40)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.a_s8vector is not None:
            oprot.writeFieldBegin('a_s8vector', TType.STRING, 14)
            oprot.writeBinary(self.a_s8vector)
            oprot.writeFieldEnd()
        if self.a_c32vector is not None:
            oprot.writeFieldBegin('a_c32vector', TType.LIST, 15)
            oprot.writeListBegin(TType.STRUCT, len(self.a_c32vector))
            for iter41 in self.a_c32vector:
                iter41.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class Knob(object):
    """
    Attributes:
     - type
     - value

    """
    thrift_spec = None


    def __init__(self, type = None, value = None,):
        self.type = type
        self.value = value

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.I32:
                    self.type = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.value = KnobBase()
                    self.value.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        self.validate()
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('Knob')
        if self.type is not None:
            oprot.writeFieldBegin('type', TType.I32, 1)
            oprot.writeI32(self.type)
            oprot.writeFieldEnd()
        if self.value is not None:
            oprot.writeFieldBegin('value', TType.STRUCT, 2)
            self.value.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class KnobProp(object):
    """
    Attributes:
     - type
     - units
     - description
     - display
     - min
     - max
     - defaultvalue

    """
    thrift_spec = None


    def __init__(self, type = None, units = None, description = None, display = None, min = None, max = None, defaultvalue = None,):
        self.type = type
        self.units = units
        self.description = description
        self.display = display
        self.min = min
        self.max = max
        self.defaultvalue = defaultvalue

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.I32:
                    self.type = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.units = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRING:
                    self.description = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.I32:
                    self.display = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.STRUCT:
                    self.min = Knob()
                    self.min.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 6:
                if ftype == TType.STRUCT:
                    self.max = Knob()
                    self.max.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 7:
                if ftype == TType.STRUCT:
                    self.defaultvalue = Knob()
                    self.defaultvalue.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        self.validate()
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('KnobProp')
        if self.type is not None:
            oprot.writeFieldBegin('type', TType.I32, 1)
            oprot.writeI32(self.type)
            oprot.writeFieldEnd()
        if self.units is not None:
            oprot.writeFieldBegin('units', TType.STRING, 2)
            oprot.writeString(self.units.encode('utf-8') if sys.version_info[0] == 2 else self.units)
            oprot.writeFieldEnd()
        if self.description is not None:
            oprot.writeFieldBegin('description', TType.STRING, 3)
            oprot.writeString(self.description.encode('utf-8') if sys.version_info[0] == 2 else self.description)
            oprot.writeFieldEnd()
        if self.display is not None:
            oprot.writeFieldBegin('display', TType.I32, 4)
            oprot.writeI32(self.display)
            oprot.writeFieldEnd()
        if self.min is not None:
            oprot.writeFieldBegin('min', TType.STRUCT, 5)
            self.min.write(oprot)
            oprot.writeFieldEnd()
        if self.max is not None:
            oprot.writeFieldBegin('max', TType.STRUCT, 6)
            self.max.write(oprot)
            oprot.writeFieldEnd()
        if self.defaultvalue is not None:
            oprot.writeFieldBegin('defaultvalue', TType.STRUCT, 7)
            self.defaultvalue.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(complex)
complex.thrift_spec = (
    None,  # 0
    (1, TType.DOUBLE, 're', None, None, ),  # 1
    (2, TType.DOUBLE, 'im', None, None, ),  # 2
)
all_structs.append(KnobBase)
KnobBase.thrift_spec = (
    None,  # 0
    (1, TType.BOOL, 'a_bool', None, None, ),  # 1
    (2, TType.BYTE, 'a_byte', None, None, ),  # 2
    (3, TType.I16, 'a_short', None, None, ),  # 3
    (4, TType.I32, 'a_int', None, None, ),  # 4
    (5, TType.I64, 'a_long', None, None, ),  # 5
    (6, TType.DOUBLE, 'a_double', None, None, ),  # 6
    (7, TType.STRING, 'a_string', 'UTF8', None, ),  # 7
    (8, TType.STRUCT, 'a_complex', [complex, None], None, ),  # 8
    (9, TType.LIST, 'a_f32vector', (TType.DOUBLE, None, False), None, ),  # 9
    (10, TType.LIST, 'a_f64vector', (TType.DOUBLE, None, False), None, ),  # 10
    (11, TType.LIST, 'a_s64vector', (TType.I64, None, False), None, ),  # 11
    (12, TType.LIST, 'a_s32vector', (TType.I32, None, False), None, ),  # 12
    (13, TType.LIST, 'a_s16vector', (TType.I16, None, False), None, ),  # 13
    (14, TType.STRING, 'a_s8vector', 'BINARY', None, ),  # 14
    (15, TType.LIST, 'a_c32vector', (TType.STRUCT, [complex, None], False), None, ),  # 15
)
all_structs.append(Knob)
Knob.thrift_spec = (
    None,  # 0
    (1, TType.I32, 'type', None, None, ),  # 1
    (2, TType.STRUCT, 'value', [KnobBase, None], None, ),  # 2
)
all_structs.append(KnobProp)
KnobProp.thrift_spec = (
    None,  # 0
    (1, TType.I32, 'type', None, None, ),  # 1
    (2, TType.STRING, 'units', 'UTF8', None, ),  # 2
    (3, TType.STRING, 'description', 'UTF8', None, ),  # 3
    (4, TType.I32, 'display', None, None, ),  # 4
    (5, TType.STRUCT, 'min', [Knob, None], None, ),  # 5
    (6, TType.STRUCT, 'max', [Knob, None], None, ),  # 6
    (7, TType.STRUCT, 'defaultvalue', [Knob, None], None, ),  # 7
)
fix_spec(all_structs)
del all_structs



#!/usr/bin/env /usr/bin/python
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

from gnuradio import gr, filter
from . import dtv_python as dtv

# FIXME move these into separate constants module
ATSC_CHANNEL_BW = 6.0e6
ATSC_SYMBOL_RATE = 4.5e6 / 286 * 684  # ~10.76 Mbaud
ATSC_RRC_SYMS = 8             # filter kernel extends over 2N+1 symbols


class atsc_rx_filter(gr.hier_block2):
    def __init__(self, input_rate, sps):
        gr.hier_block2.__init__(self, "atsc_rx_filter",
                                # Input signature
                                gr.io_signature(1, 1, gr.sizeof_gr_complex),
                                gr.io_signature(1, 1, gr.sizeof_gr_complex))  # Output signature

        # Create matched RX filter with RRC response for fractional
        # interpolator.
        nfilts = 16
        output_rate = ATSC_SYMBOL_RATE * sps  # Desired oversampled sample rate
        filter_rate = input_rate * nfilts
        symbol_rate = ATSC_SYMBOL_RATE / 2.0  # One-sided bandwidth of sideband
        # 1.0-(0.5*ATSC_SYMBOL_RATE/ATSC_CHANNEL_BW) # ~10.3%
        excess_bw = 0.1152
        ntaps = int((2 * ATSC_RRC_SYMS + 1) * sps * nfilts)
        interp = output_rate / input_rate
        gain = nfilts * symbol_rate / filter_rate
        rrc_taps = filter.firdes.root_raised_cosine(gain,             # Filter gain
                                                    filter_rate,      # PFB filter prototype rate
                                                    symbol_rate,      # ATSC symbol rate
                                                    excess_bw,        # ATSC RRC excess bandwidth
                                                    ntaps)            # Length of filter

        pfb = filter.pfb_arb_resampler_ccf(interp, rrc_taps, nfilts)

        # Connect pipeline
        self.connect(self, pfb, self)



#!/usr/bin/env python
##################################################
# Gnuradio Python Flow Graph
# Title: IQ Phase Balancer
# Author: matt@ettus.com
# Generated: Thu Aug  1 11:49:41 2013
##################################################

from gnuradio import blocks
from gnuradio import filter
from gnuradio import gr
from gnuradio.filter import firdes


class phase_bal(gr.hier_block2):

    def __init__(self, alpha=0):
        gr.hier_block2.__init__(
            self, "IQ Phase Balancer",
            gr.io_signature(1, 1, gr.sizeof_gr_complex * 1),
            gr.io_signature(1, 1, gr.sizeof_gr_complex * 1),
        )

        ##################################################
        # Parameters
        ##################################################
        self.alpha = alpha

        ##################################################
        # Blocks
        ##################################################
        self.filter_single_pole_iir_filter_xx_0 = filter.single_pole_iir_filter_ff(
            alpha, 1)
        self.blocks_sub_xx_1 = blocks.sub_ff(1)
        self.blocks_sub_xx_0 = blocks.sub_ff(1)
        self.blocks_multiply_xx_2 = blocks.multiply_vff(1)
        self.blocks_multiply_xx_1 = blocks.multiply_vff(1)
        self.blocks_multiply_xx_0 = blocks.multiply_vff(1)
        self.blocks_multiply_const_vxx_0 = blocks.multiply_const_vff((2, ))
        self.blocks_float_to_complex_0 = blocks.float_to_complex(1)
        self.blocks_divide_xx_0 = blocks.divide_ff(1)
        self.blocks_complex_to_mag_squared_0 = blocks.complex_to_mag_squared(1)
        self.blocks_complex_to_float_0 = blocks.complex_to_float(1)

        ##################################################
        # Connections
        ##################################################
        self.connect((self.blocks_complex_to_float_0, 0),
                     (self.blocks_multiply_xx_0, 0))
        self.connect((self.blocks_complex_to_float_0, 1),
                     (self.blocks_multiply_xx_0, 1))
        self.connect((self.blocks_multiply_xx_0, 0),
                     (self.blocks_divide_xx_0, 0))
        self.connect((self.blocks_sub_xx_0, 0),
                     (self.blocks_float_to_complex_0, 1))
        self.connect((self.blocks_multiply_xx_1, 0), (self.blocks_sub_xx_0, 1))
        self.connect((self.filter_single_pole_iir_filter_xx_0, 0),
                     (self.blocks_multiply_xx_1, 1))
        self.connect((self.blocks_complex_to_float_0, 0),
                     (self.blocks_multiply_xx_1, 0))
        self.connect((self.blocks_multiply_xx_2, 0), (self.blocks_sub_xx_1, 1))
        self.connect((self.blocks_complex_to_float_0, 1),
                     (self.blocks_sub_xx_0, 0))
        self.connect((self.blocks_sub_xx_1, 0),
                     (self.blocks_float_to_complex_0, 0))
        self.connect((self.blocks_complex_to_mag_squared_0, 0),
                     (self.blocks_divide_xx_0, 1))
        self.connect((self.blocks_complex_to_float_0, 0),
                     (self.blocks_sub_xx_1, 0))
        self.connect((self.blocks_divide_xx_0, 0),
                     (self.blocks_multiply_const_vxx_0, 0))
        self.connect((self.blocks_multiply_const_vxx_0, 0),
                     (self.filter_single_pole_iir_filter_xx_0, 0))
        self.connect((self, 0), (self.blocks_complex_to_float_0, 0))
        self.connect((self, 0), (self.blocks_complex_to_mag_squared_0, 0))
        self.connect((self.blocks_float_to_complex_0, 0), (self, 0))
        self.connect((self.filter_single_pole_iir_filter_xx_0, 0),
                     (self.blocks_multiply_xx_2, 0))
        self.connect((self.blocks_complex_to_float_0, 1),
                     (self.blocks_multiply_xx_2, 1))

    def get_alpha(self):
        return self.alpha

    def set_alpha(self, alpha):
        self.alpha = alpha
        self.filter_single_pole_iir_filter_xx_0.set_taps(self.alpha)



#
# Copyright 2018 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
""" Create a whole new out-of-tree module """


import re
import os

import click

from gnuradio import gr
from ..core import ModToolNewModule, validate_name
from .base import common_params, run, cli_input, ModToolException


@click.command('newmod', short_help=ModToolNewModule.description)
@click.option('--srcdir',
              type=click.Path(file_okay=False, dir_okay=True, readable=True),
              help="Source directory for the module template.")
@common_params
@click.argument('module_name', metavar="MODULE-NAME", nargs=1, required=False)
def cli(**kwargs):
    """
    \b
    Create a new out-of-tree module

    The argument MODULE-NAME is the name of the module to be added.
    """
    kwargs['cli'] = True
    self = ModToolNewModule(**kwargs)
    get_modname(self)
    self.dir = os.path.join(self.dir, f'gr-{self.info["modname"]}')
    try:
        os.stat(self.dir)
    except OSError:
        pass  # This is what should happen
    else:
        raise ModToolException('The given directory exists.')
    if self.srcdir is None:
        self.srcdir = os.path.join(
            gr.prefix(), 'share', 'gnuradio', 'modtool', 'templates', 'gr-newmod')
    if not os.path.isdir(self.srcdir):
        raise ModToolException('Could not find gr-newmod source dir.')
    run(self)


def get_modname(self):
    """ Get the name of the new module to be added """
    if self.info['modname'] is None:
        while not self.info['modname'] or self.info['modname'].isspace():
            self.info['modname'] = cli_input('Name of the new module: ')
    validate_name('module', self.info['modname'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2023 Ettus Research, a National Instruments Brand.
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from PyQt5 import Qt
from gnuradio import gr
import pmt


class ReplayMsgPushButton(gr.sync_block, Qt.QPushButton):
    """
    This block creates a variable push button that creates a message
    when clicked. The message will be formatted as a dictionary to pass
    to the RFNoC Replay block
    """

    def __init__(self, lbl, relBackColor, relFontColor,
                 command, port, offset=-1, size=-1, time=-1, repeat=False):
        gr.sync_block.__init__(self, name="ReplayMsgPushButton",
                               in_sig=None, out_sig=None)
        Qt.QPushButton.__init__(self, lbl)

        self.lbl = lbl

        self.replayDict = {'command': command}
        self.replayDict['port'] = port
        self.replayDict['repeat'] = repeat
        # If the user does not specify a command offset, size, or time,
        # don't add to the command. The properties will be used for
        # offset and size, and the command will happen immediately.
        if offset != -1:
            self.replayDict['offset'] = offset
        if size != -1:
            self.replayDict['size'] = size
        if time != -1:
            self.replayDict['time'] = time

        styleStr = ""
        if relBackColor != 'default':
            styleStr = "background-color: " + relBackColor + "; "

        if relFontColor:
            styleStr += "color: " + relFontColor + "; "

        self.setStyleSheet(styleStr)

        self.clicked[bool].connect(self.onBtnClicked)

        self.message_port_register_out(pmt.intern("pressed"))

    def set_command(self, command):
        self.replayDict['command'] = command

    def set_port(self, port):
        self.replayDict['port'] = port

    def set_size(self, size):
        self.replayDict['size'] = size

    def set_offset(self, offset):
        self.replayDict['offset'] = offset

    def set_time(self, time):
        self.replayDict['time'] = time

    def set_repeat(self, repeat):
        self.replayDict['repeat'] = repeat

    def onBtnClicked(self, pressed):
        self.message_port_pub(pmt.intern("pressed"), pmt.to_pmt(self.replayDict))



#
# Copyright 2019 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
""" Base class for the modules """


from abc import ABC, abstractmethod


class BlockToolException(Exception):
    """ Standard exception for blocktool classes. """
    pass


class BlockTool(ABC):
    """ Base class for all blocktool command classes. """
    name = 'base'
    description = None

    def __init__(self, modname=None, filename=None, targetdir=None,
                 target_file=None, module=None, impldir=None, impl_file=None,
                 yaml=False, json=False, include_paths=None, **kwargs):
        """ __init__ """
        pass

    def _validate(self):
        """ Validates the arguments """
        pass

    @abstractmethod
    def run_blocktool(self):
        """ Override this. """
        pass



#!/usr/bin/env python
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, blocks

from . import fec_python as fec

from .bitflip import read_bitlist


class extended_tagged_encoder(gr.hier_block2):
    def __init__(self, encoder_obj_list, puncpat=None, lentagname=None, mtu=1500):
        gr.hier_block2.__init__(self, "extended_tagged_encoder",
                                gr.io_signature(1, 1, gr.sizeof_char),
                                gr.io_signature(1, 1, gr.sizeof_char))

        self.blocks = []
        self.puncpat = puncpat

        # If it's a list of encoders, take the first one, unless it's
        # a list of lists of encoders.
        if(type(encoder_obj_list) == list):
            # This block doesn't handle parallelism of > 1
            # We could just grab encoder [0][0], but we don't want to encourage this.
            if(type(encoder_obj_list[0]) == list):
                gr.log.info(
                    "fec.extended_tagged_encoder: Parallelism must be 0 or 1.")
                raise AttributeError

            encoder_obj = encoder_obj_list[0]

        # Otherwise, just take it as is
        else:
            encoder_obj = encoder_obj_list

        # If lentagname is None, fall back to using the non tagged
        # stream version
        if type(lentagname) == str:
            if(lentagname.lower() == 'none'):
                lentagname = None

        if fec.get_encoder_input_conversion(encoder_obj) == "pack":
            self.blocks.append(blocks.pack_k_bits_bb(8))

        if(not lentagname):
            self.blocks.append(fec.encoder(encoder_obj,
                                           gr.sizeof_char,
                                           gr.sizeof_char))
        else:
            self.blocks.append(fec.tagged_encoder(encoder_obj,
                                                  gr.sizeof_char,
                                                  gr.sizeof_char,
                                                  lentagname, mtu))

        if self.puncpat != '11':
            self.blocks.append(fec.puncture_bb(
                len(puncpat), read_bitlist(puncpat), 0))

        # Connect the input to the encoder and the output to the
        # puncture if used or the encoder if not.
        self.connect((self, 0), (self.blocks[0], 0))
        self.connect((self.blocks[-1], 0), (self, 0))

        # If using the puncture block, add it into the flowgraph after
        # the encoder.
        for i in range(len(self.blocks) - 1):
            self.connect((self.blocks[i], 0), (self.blocks[i + 1], 0))



#
# Autogenerated by Thrift Compiler (0.22.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec
from uuid import UUID

import sys
import logging
from .ttypes import *
from thrift.Thrift import TProcessor
from thrift.transport import TTransport
all_structs = []


class Iface(object):
    def setKnobs(self, knobs):
        """
        Parameters:
         - knobs

        """
        pass

    def getKnobs(self, knobs):
        """
        Parameters:
         - knobs

        """
        pass

    def getRe(self, knobs):
        """
        Parameters:
         - knobs

        """
        pass

    def properties(self, knobs):
        """
        Parameters:
         - knobs

        """
        pass

    def postMessage(self, blk_alias, port, msg):
        """
        Parameters:
         - blk_alias
         - port
         - msg

        """
        pass

    def shutdown(self):
        pass


class Client(Iface):
    def __init__(self, iprot, oprot=None):
        self._iprot = self._oprot = iprot
        if oprot is not None:
            self._oprot = oprot
        self._seqid = 0

    def setKnobs(self, knobs):
        """
        Parameters:
         - knobs

        """
        self.send_setKnobs(knobs)
        self.recv_setKnobs()

    def send_setKnobs(self, knobs):
        self._oprot.writeMessageBegin('setKnobs', TMessageType.CALL, self._seqid)
        args = setKnobs_args()
        args.knobs = knobs
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_setKnobs(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = setKnobs_result()
        result.read(iprot)
        iprot.readMessageEnd()
        return

    def getKnobs(self, knobs):
        """
        Parameters:
         - knobs

        """
        self.send_getKnobs(knobs)
        return self.recv_getKnobs()

    def send_getKnobs(self, knobs):
        self._oprot.writeMessageBegin('getKnobs', TMessageType.CALL, self._seqid)
        args = getKnobs_args()
        args.knobs = knobs
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_getKnobs(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = getKnobs_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        raise TApplicationException(TApplicationException.MISSING_RESULT, "getKnobs failed: unknown result")

    def getRe(self, knobs):
        """
        Parameters:
         - knobs

        """
        self.send_getRe(knobs)
        return self.recv_getRe()

    def send_getRe(self, knobs):
        self._oprot.writeMessageBegin('getRe', TMessageType.CALL, self._seqid)
        args = getRe_args()
        args.knobs = knobs
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_getRe(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = getRe_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        raise TApplicationException(TApplicationException.MISSING_RESULT, "getRe failed: unknown result")

    def properties(self, knobs):
        """
        Parameters:
         - knobs

        """
        self.send_properties(knobs)
        return self.recv_properties()

    def send_properties(self, knobs):
        self._oprot.writeMessageBegin('properties', TMessageType.CALL, self._seqid)
        args = properties_args()
        args.knobs = knobs
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_properties(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = properties_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        raise TApplicationException(TApplicationException.MISSING_RESULT, "properties failed: unknown result")

    def postMessage(self, blk_alias, port, msg):
        """
        Parameters:
         - blk_alias
         - port
         - msg

        """
        self.send_postMessage(blk_alias, port, msg)
        self.recv_postMessage()

    def send_postMessage(self, blk_alias, port, msg):
        self._oprot.writeMessageBegin('postMessage', TMessageType.CALL, self._seqid)
        args = postMessage_args()
        args.blk_alias = blk_alias
        args.port = port
        args.msg = msg
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_postMessage(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = postMessage_result()
        result.read(iprot)
        iprot.readMessageEnd()
        return

    def shutdown(self):
        self.send_shutdown()
        self.recv_shutdown()

    def send_shutdown(self):
        self._oprot.writeMessageBegin('shutdown', TMessageType.CALL, self._seqid)
        args = shutdown_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_shutdown(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = shutdown_result()
        result.read(iprot)
        iprot.readMessageEnd()
        return


class Processor(Iface, TProcessor):
    def __init__(self, handler):
        self._handler = handler
        self._processMap = {}
        self._processMap["setKnobs"] = Processor.process_setKnobs
        self._processMap["getKnobs"] = Processor.process_getKnobs
        self._processMap["getRe"] = Processor.process_getRe
        self._processMap["properties"] = Processor.process_properties
        self._processMap["postMessage"] = Processor.process_postMessage
        self._processMap["shutdown"] = Processor.process_shutdown
        self._on_message_begin = None

    def on_message_begin(self, func):
        self._on_message_begin = func

    def process(self, iprot, oprot):
        (name, type, seqid) = iprot.readMessageBegin()
        if self._on_message_begin:
            self._on_message_begin(name, type, seqid)
        if name not in self._processMap:
            iprot.skip(TType.STRUCT)
            iprot.readMessageEnd()
            x = TApplicationException(TApplicationException.UNKNOWN_METHOD, 'Unknown function %s' % (name))
            oprot.writeMessageBegin(name, TMessageType.EXCEPTION, seqid)
            x.write(oprot)
            oprot.writeMessageEnd()
            oprot.trans.flush()
            return
        else:
            self._processMap[name](self, seqid, iprot, oprot)
        return True

    def process_setKnobs(self, seqid, iprot, oprot):
        args = setKnobs_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = setKnobs_result()
        try:
            self._handler.setKnobs(args.knobs)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("setKnobs", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_getKnobs(self, seqid, iprot, oprot):
        args = getKnobs_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = getKnobs_result()
        try:
            result.success = self._handler.getKnobs(args.knobs)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("getKnobs", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_getRe(self, seqid, iprot, oprot):
        args = getRe_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = getRe_result()
        try:
            result.success = self._handler.getRe(args.knobs)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("getRe", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_properties(self, seqid, iprot, oprot):
        args = properties_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = properties_result()
        try:
            result.success = self._handler.properties(args.knobs)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("properties", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_postMessage(self, seqid, iprot, oprot):
        args = postMessage_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = postMessage_result()
        try:
            self._handler.postMessage(args.blk_alias, args.port, args.msg)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("postMessage", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_shutdown(self, seqid, iprot, oprot):
        args = shutdown_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = shutdown_result()
        try:
            self._handler.shutdown()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("shutdown", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

# HELPER FUNCTIONS AND STRUCTURES


class setKnobs_args(object):
    """
    Attributes:
     - knobs

    """
    thrift_spec = None


    def __init__(self, knobs = None,):
        self.knobs = knobs

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.MAP:
                    self.knobs = {}
                    (_ktype43, _vtype44, _size42) = iprot.readMapBegin()
                    for _i46 in range(_size42):
                        _key47 = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                        _val48 = Knob()
                        _val48.read(iprot)
                        self.knobs[_key47] = _val48
                    iprot.readMapEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        self.validate()
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('setKnobs_args')
        if self.knobs is not None:
            oprot.writeFieldBegin('knobs', TType.MAP, 1)
            oprot.writeMapBegin(TType.STRING, TType.STRUCT, len(self.knobs))
            for kiter49, viter50 in self.knobs.items():
                oprot.writeString(kiter49.encode('utf-8') if sys.version_info[0] == 2 else kiter49)
                viter50.write(oprot)
            oprot.writeMapEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(setKnobs_args)
setKnobs_args.thrift_spec = (
    None,  # 0
    (1, TType.MAP, 'knobs', (TType.STRING, 'UTF8', TType.STRUCT, [Knob, None], False), None, ),  # 1
)


class setKnobs_result(object):
    thrift_spec = None


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        self.validate()
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('setKnobs_result')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(setKnobs_result)
setKnobs_result.thrift_spec = (
)


class getKnobs_args(object):
    """
    Attributes:
     - knobs

    """
    thrift_spec = None


    def __init__(self, knobs = None,):
        self.knobs = knobs

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.LIST:
                    self.knobs = []
                    (_etype54, _size51) = iprot.readListBegin()
                    for _i55 in range(_size51):
                        _elem56 = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                        self.knobs.append(_elem56)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        self.validate()
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getKnobs_args')
        if self.knobs is not None:
            oprot.writeFieldBegin('knobs', TType.LIST, 1)
            oprot.writeListBegin(TType.STRING, len(self.knobs))
            for iter57 in self.knobs:
                oprot.writeString(iter57.encode('utf-8') if sys.version_info[0] == 2 else iter57)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getKnobs_args)
getKnobs_args.thrift_spec = (
    None,  # 0
    (1, TType.LIST, 'knobs', (TType.STRING, 'UTF8', False), None, ),  # 1
)


class getKnobs_result(object):
    """
    Attributes:
     - success

    """
    thrift_spec = None


    def __init__(self, success = None,):
        self.success = success

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.MAP:
                    self.success = {}
                    (_ktype59, _vtype60, _size58) = iprot.readMapBegin()
                    for _i62 in range(_size58):
                        _key63 = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                        _val64 = Knob()
                        _val64.read(iprot)
                        self.success[_key63] = _val64
                    iprot.readMapEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        self.validate()
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getKnobs_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.MAP, 0)
            oprot.writeMapBegin(TType.STRING, TType.STRUCT, len(self.success))
            for kiter65, viter66 in self.success.items():
                oprot.writeString(kiter65.encode('utf-8') if sys.version_info[0] == 2 else kiter65)
                viter66.write(oprot)
            oprot.writeMapEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getKnobs_result)
getKnobs_result.thrift_spec = (
    (0, TType.MAP, 'success', (TType.STRING, 'UTF8', TType.STRUCT, [Knob, None], False), None, ),  # 0
)


class getRe_args(object):
    """
    Attributes:
     - knobs

    """
    thrift_spec = None


    def __init__(self, knobs = None,):
        self.knobs = knobs

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.LIST:
                    self.knobs = []
                    (_etype70, _size67) = iprot.readListBegin()
                    for _i71 in range(_size67):
                        _elem72 = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                        self.knobs.append(_elem72)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        self.validate()
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getRe_args')
        if self.knobs is not None:
            oprot.writeFieldBegin('knobs', TType.LIST, 1)
            oprot.writeListBegin(TType.STRING, len(self.knobs))
            for iter73 in self.knobs:
                oprot.writeString(iter73.encode('utf-8') if sys.version_info[0] == 2 else iter73)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getRe_args)
getRe_args.thrift_spec = (
    None,  # 0
    (1, TType.LIST, 'knobs', (TType.STRING, 'UTF8', False), None, ),  # 1
)


class getRe_result(object):
    """
    Attributes:
     - success

    """
    thrift_spec = None


    def __init__(self, success = None,):
        self.success = success

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.MAP:
                    self.success = {}
                    (_ktype75, _vtype76, _size74) = iprot.readMapBegin()
                    for _i78 in range(_size74):
                        _key79 = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                        _val80 = Knob()
                        _val80.read(iprot)
                        self.success[_key79] = _val80
                    iprot.readMapEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        self.validate()
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getRe_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.MAP, 0)
            oprot.writeMapBegin(TType.STRING, TType.STRUCT, len(self.success))
            for kiter81, viter82 in self.success.items():
                oprot.writeString(kiter81.encode('utf-8') if sys.version_info[0] == 2 else kiter81)
                viter82.write(oprot)
            oprot.writeMapEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getRe_result)
getRe_result.thrift_spec = (
    (0, TType.MAP, 'success', (TType.STRING, 'UTF8', TType.STRUCT, [Knob, None], False), None, ),  # 0
)


class properties_args(object):
    """
    Attributes:
     - knobs

    """
    thrift_spec = None


    def __init__(self, knobs = None,):
        self.knobs = knobs

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.LIST:
                    self.knobs = []
                    (_etype86, _size83) = iprot.readListBegin()
                    for _i87 in range(_size83):
                        _elem88 = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                        self.knobs.append(_elem88)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        self.validate()
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('properties_args')
        if self.knobs is not None:
            oprot.writeFieldBegin('knobs', TType.LIST, 1)
            oprot.writeListBegin(TType.STRING, len(self.knobs))
            for iter89 in self.knobs:
                oprot.writeString(iter89.encode('utf-8') if sys.version_info[0] == 2 else iter89)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(properties_args)
properties_args.thrift_spec = (
    None,  # 0
    (1, TType.LIST, 'knobs', (TType.STRING, 'UTF8', False), None, ),  # 1
)


class properties_result(object):
    """
    Attributes:
     - success

    """
    thrift_spec = None


    def __init__(self, success = None,):
        self.success = success

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.MAP:
                    self.success = {}
                    (_ktype91, _vtype92, _size90) = iprot.readMapBegin()
                    for _i94 in range(_size90):
                        _key95 = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                        _val96 = KnobProp()
                        _val96.read(iprot)
                        self.success[_key95] = _val96
                    iprot.readMapEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        self.validate()
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('properties_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.MAP, 0)
            oprot.writeMapBegin(TType.STRING, TType.STRUCT, len(self.success))
            for kiter97, viter98 in self.success.items():
                oprot.writeString(kiter97.encode('utf-8') if sys.version_info[0] == 2 else kiter97)
                viter98.write(oprot)
            oprot.writeMapEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(properties_result)
properties_result.thrift_spec = (
    (0, TType.MAP, 'success', (TType.STRING, 'UTF8', TType.STRUCT, [KnobProp, None], False), None, ),  # 0
)


class postMessage_args(object):
    """
    Attributes:
     - blk_alias
     - port
     - msg

    """
    thrift_spec = None


    def __init__(self, blk_alias = None, port = None, msg = None,):
        self.blk_alias = blk_alias
        self.port = port
        self.msg = msg

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.blk_alias = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.port = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRING:
                    self.msg = iprot.readBinary()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        self.validate()
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('postMessage_args')
        if self.blk_alias is not None:
            oprot.writeFieldBegin('blk_alias', TType.STRING, 1)
            oprot.writeString(self.blk_alias.encode('utf-8') if sys.version_info[0] == 2 else self.blk_alias)
            oprot.writeFieldEnd()
        if self.port is not None:
            oprot.writeFieldBegin('port', TType.STRING, 2)
            oprot.writeString(self.port.encode('utf-8') if sys.version_info[0] == 2 else self.port)
            oprot.writeFieldEnd()
        if self.msg is not None:
            oprot.writeFieldBegin('msg', TType.STRING, 3)
            oprot.writeBinary(self.msg)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(postMessage_args)
postMessage_args.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'blk_alias', 'UTF8', None, ),  # 1
    (2, TType.STRING, 'port', 'UTF8', None, ),  # 2
    (3, TType.STRING, 'msg', 'BINARY', None, ),  # 3
)


class postMessage_result(object):
    thrift_spec = None


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        self.validate()
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('postMessage_result')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(postMessage_result)
postMessage_result.thrift_spec = (
)


class shutdown_args(object):
    thrift_spec = None


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        self.validate()
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('shutdown_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(shutdown_args)
shutdown_args.thrift_spec = (
)


class shutdown_result(object):
    thrift_spec = None


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        self.validate()
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('shutdown_result')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(shutdown_result)
shutdown_result.thrift_spec = (
)
fix_spec(all_structs)
del all_structs



#!/usr/bin/env python
##################################################
# Gnuradio Python Flow Graph
# Title: Conj FS IQBal
# Author: matt@ettus.com
# Description: Frequency Selective Conjugate Method IQ Balance Corrector
# Generated: Thu Aug  1 13:00:27 2013
##################################################

from gnuradio import blocks
from gnuradio import filter
from gnuradio import gr
from gnuradio.filter import firdes


class conj_fs_iqcorr(gr.hier_block2):

    def __init__(self, delay=0, taps=[]):
        gr.hier_block2.__init__(
            self, "Conj FS IQBal",
            gr.io_signature(1, 1, gr.sizeof_gr_complex * 1),
            gr.io_signature(1, 1, gr.sizeof_gr_complex * 1),
        )

        ##################################################
        # Parameters
        ##################################################
        self.delay = delay
        self.taps = taps

        ##################################################
        # Blocks
        ##################################################
        self.filter_fir_filter_xxx_0 = filter.fir_filter_ccc(1, (taps))
        self.delay_0 = blocks.delay(gr.sizeof_gr_complex * 1, delay)
        self.blocks_conjugate_cc_0 = blocks.conjugate_cc()
        self.blocks_add_xx_0 = blocks.add_vcc(1)

        ##################################################
        # Connections
        ##################################################
        self.connect((self.blocks_add_xx_0, 0), (self, 0))
        self.connect((self, 0), (self.blocks_conjugate_cc_0, 0))
        self.connect((self.filter_fir_filter_xxx_0, 0),
                     (self.blocks_add_xx_0, 1))
        self.connect((self.blocks_conjugate_cc_0, 0),
                     (self.filter_fir_filter_xxx_0, 0))
        self.connect((self, 0), (self.delay_0, 0))
        self.connect((self.delay_0, 0), (self.blocks_add_xx_0, 0))

    def get_delay(self):
        return self.delay

    def set_delay(self, delay):
        self.delay = delay
        self.delay_0.set_dly(self.delay)

    def get_taps(self):
        return self.taps

    def set_taps(self, taps):
        self.taps = taps
        self.filter_fir_filter_xxx_0.set_taps((self.taps))



#
# Copyright 2005,2006,2011 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

"""
QPSK modulation.

Demodulation is not included since the generic_mod_demod
"""

from gnuradio import gr
from gnuradio.digital.generic_mod_demod import generic_mod, generic_demod
from gnuradio.digital.generic_mod_demod import shared_mod_args, shared_demod_args
from .utils import mod_codes
from . import digital_python as digital
from . import modulation_utils

# The default encoding (e.g. gray-code, set-partition)
_def_mod_code = mod_codes.GRAY_CODE

# /////////////////////////////////////////////////////////////////////////////
#                           QPSK constellation
# /////////////////////////////////////////////////////////////////////////////


def qpsk_constellation(mod_code=_def_mod_code):
    """
    Creates a QPSK constellation.
    """
    if mod_code != mod_codes.GRAY_CODE:
        raise ValueError(
            "This QPSK mod/demod works only for gray-coded constellations.")
    return digital.constellation_qpsk()

# /////////////////////////////////////////////////////////////////////////////
#                           DQPSK constellation
# /////////////////////////////////////////////////////////////////////////////


def dqpsk_constellation(mod_code=_def_mod_code):
    if mod_code != mod_codes.GRAY_CODE:
        raise ValueError(
            "The DQPSK constellation is only generated for gray_coding.  But it can be used for non-grayed coded modulation if one doesn't use the pre-differential code.")
    return digital.constellation_dqpsk()


#
# Add these to the mod/demod registry
#
modulation_utils.add_type_1_constellation('qpsk', qpsk_constellation)
modulation_utils.add_type_1_constellation('dqpsk', dqpsk_constellation)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2007-2020 Free Software Foundation, Inc.
#
# SPDX-License-Identifier: GPL-3.0
#
# GNU Radio Python Flow Graph
# Title: FM stereo demod block

import math

from gnuradio import gr
from gnuradio import blocks
from gnuradio import filter, fft
from gnuradio.filter import firdes
from gnuradio import analog


class wfm_rcv_pll(gr.hier_block2):
    def __init__(self, demod_rate, audio_decimation, deemph_tau):
        """
        Hierarchical block for demodulating a broadcast FM signal.

        The input is the downconverted complex baseband signal (gr_complex).
        The output is two streams of the demodulated audio (float) 0=Left, 1=Right.

        Args:
            demod_rate: input sample rate of complex baseband input. (float)
            audio_decimation: how much to decimate demod_rate to get to audio. (integer)
            deemph_tau: deemphasis ime constant in seconds (75us in US, 50us in EUR). (float)
        """
        gr.hier_block2.__init__(self, "wfm_rcv_pll",
                                # Input signature
                                gr.io_signature(1, 1, gr.sizeof_gr_complex),
                                gr.io_signature(2, 2, gr.sizeof_float))      # Output signature

        if audio_decimation != int(audio_decimation):
            raise ValueError("audio_decimation needs to be an integer")
        audio_decimation = int(audio_decimation)

        ##################################################
        # Variables
        ##################################################
        self.demod_rate = demod_rate
        self.deemph_tau = deemph_tau
        self.stereo_carrier_filter_coeffs = stereo_carrier_filter_coeffs = firdes.band_pass(
            -2.0, demod_rate, 37600, 38400, 400, fft.window.WIN_HAMMING, 6.76)
        self.pilot_carrier_filter_coeffs = pilot_carrier_filter_coeffs = firdes.complex_band_pass(
            1.0, demod_rate, 18980, 19020, 1500, fft.window.WIN_HAMMING, 6.76)
        self.deviation = deviation = 75000
        self.audio_filter_coeffs = audio_filter_coeffs = firdes.low_pass(
            1, demod_rate, 15000, 1500, fft.window.WIN_HAMMING, 6.76)
        self.audio_decim = audio_decim = audio_decimation
        self.audio_rate = audio_rate = demod_rate / audio_decim
        self.samp_delay = samp_delay = (len(
            pilot_carrier_filter_coeffs) - 1) // 2 + (len(stereo_carrier_filter_coeffs) - 1) // 2

        ##################################################
        # Blocks
        ##################################################
        self.pilot_carrier_bpf = filter.fir_filter_fcc(
            1, pilot_carrier_filter_coeffs)
        self.pilot_carrier_bpf.declare_sample_delay(0)
        self.stereo_carrier_bpf = filter.fft_filter_fff(
            1, stereo_carrier_filter_coeffs, 1)
        self.stereo_carrier_bpf.declare_sample_delay(0)
        self.stereo_audio_lpf = filter.fft_filter_fff(
            audio_decim, audio_filter_coeffs, 1)
        self.stereo_audio_lpf.declare_sample_delay(0)
        self.mono_audio_lpf = filter.fft_filter_fff(
            audio_decim, audio_filter_coeffs, 1)
        self.mono_audio_lpf.declare_sample_delay(0)
        self.blocks_stereo_multiply = blocks.multiply_ff(1)
        self.blocks_pilot_multiply = blocks.multiply_cc(1)
        self.blocks_complex_to_imag = blocks.complex_to_imag(1)
        self.blocks_right_sub = blocks.sub_ff(1)
        self.blocks_left_add = blocks.add_ff(1)
        self.analog_quadrature_demod_cf = analog.quadrature_demod_cf(
            demod_rate / (2 * math.pi * deviation))
        self.analog_pll_refout_cc = analog.pll_refout_cc(
            0.001, 2 * math.pi * 19200 / demod_rate, 2 * math.pi * 18800 / demod_rate)
        self.analog_right_fm_deemph = analog.fm_deemph(
            fs=audio_rate, tau=deemph_tau)
        self.analog_left_fm_deemph = analog.fm_deemph(
            fs=audio_rate, tau=deemph_tau)
        self.blocks_delay_0 = blocks.delay(gr.sizeof_float * 1, samp_delay)

        ##################################################
        # Connections
        ##################################################
        self.connect((self.analog_left_fm_deemph, 0), (self, 0))
        self.connect((self.analog_right_fm_deemph, 0), (self, 1))
        self.connect((self.analog_pll_refout_cc, 0),
                     (self.blocks_pilot_multiply, 1))
        self.connect((self.analog_pll_refout_cc, 0),
                     (self.blocks_pilot_multiply, 0))
        self.connect((self.analog_quadrature_demod_cf, 0),
                     (self.blocks_delay_0, 0))
        self.connect((self.blocks_delay_0, 0),
                     (self.blocks_stereo_multiply, 0))
        self.connect((self.blocks_delay_0, 0), (self.mono_audio_lpf, 0))
        self.connect((self.analog_quadrature_demod_cf, 0),
                     (self.pilot_carrier_bpf, 0))
        self.connect((self.blocks_left_add, 0),
                     (self.analog_left_fm_deemph, 0))
        self.connect((self.blocks_right_sub, 0),
                     (self.analog_right_fm_deemph, 0))
        self.connect((self.blocks_complex_to_imag, 0),
                     (self.stereo_carrier_bpf, 0))
        self.connect((self.blocks_pilot_multiply, 0),
                     (self.blocks_complex_to_imag, 0))
        self.connect((self.blocks_stereo_multiply, 0),
                     (self.stereo_audio_lpf, 0))  # L - R path
        self.connect((self.mono_audio_lpf, 0), (self.blocks_left_add, 1))
        self.connect((self.mono_audio_lpf, 0), (self.blocks_right_sub, 0))
        self.connect((self.stereo_audio_lpf, 0), (self.blocks_left_add, 0))
        self.connect((self.stereo_audio_lpf, 0), (self.blocks_right_sub, 1))
        self.connect((self.stereo_carrier_bpf, 0),
                     (self.blocks_stereo_multiply, 1))
        self.connect((self.pilot_carrier_bpf, 0),
                     (self.analog_pll_refout_cc, 0))
        self.connect((self, 0), (self.analog_quadrature_demod_cf, 0))



#!/usr/bin/env python
#
# Copyright 2011 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

class GrayCodeGenerator(object):
    """
    Generates and caches gray codes.
    """

    def __init__(self):
        self.gcs = [0, 1]
        # The last power of two passed through.
        self.lp2 = 2
        # The next power of two that will be passed through.
        self.np2 = 4
        # Current index
        self.i = 2

    def get_gray_code(self, length):
        """
        Returns a list of gray code of given length.
        """
        if len(self.gcs) < length:
            self.generate_new_gray_code(length)
        return self.gcs[:length]

    def generate_new_gray_code(self, length):
        """
        Generates new gray code and places into cache.
        """
        while len(self.gcs) < length:
            if self.i == self.lp2:
                # if i is a power of two then gray number is of form 1100000...
                result = self.i + self.i // 2
            else:
                # if not we take advantage of the symmetry of all but the last bit
                # around a power of two.
                result = self.gcs[2 * self.lp2 - 1 - self.i] + self.lp2
            self.gcs.append(result)
            self.i += 1
            if self.i == self.np2:
                self.lp2 = self.i
                self.np2 = self.i * 2


_gray_code_generator = GrayCodeGenerator()

gray_code = _gray_code_generator.get_gray_code



#!/usr/bin/env python
#
# Copyright 2008,2009 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

"""
Abstract GNU Radio publisher/subscriber interface

This is a proof of concept implementation, will likely change significantly.
"""


class pubsub(dict):
    def __init__(self):
        self._publishers = {}
        self._subscribers = {}
        self._proxies = {}

    def __missing__(self, key, value=None):
        dict.__setitem__(self, key, value)
        self._publishers[key] = None
        self._subscribers[key] = []
        self._proxies[key] = None

    def __setitem__(self, key, val):
        if key not in self:
            self.__missing__(key, val)
        elif self._proxies[key] is not None:
            (p, newkey) = self._proxies[key]
            p[newkey] = val
        else:
            dict.__setitem__(self, key, val)
        for sub in self._subscribers[key]:
            # Note this means subscribers will get called in the thread
            # context of the 'set' caller.
            sub(val)

    def __getitem__(self, key):
        if key not in self:
            self.__missing__(key)
        if self._proxies[key] is not None:
            (p, newkey) = self._proxies[key]
            return p[newkey]
        elif self._publishers[key] is not None:
            return self._publishers[key]()
        else:
            return dict.__getitem__(self, key)

    def publish(self, key, publisher):
        if key not in self:
            self.__missing__(key)
        if self._proxies[key] is not None:
            (p, newkey) = self._proxies[key]
            p.publish(newkey, publisher)
        else:
            self._publishers[key] = publisher

    def subscribe(self, key, subscriber):
        if key not in self:
            self.__missing__(key)
        if self._proxies[key] is not None:
            (p, newkey) = self._proxies[key]
            p.subscribe(newkey, subscriber)
        else:
            self._subscribers[key].append(subscriber)

    def unpublish(self, key):
        if self._proxies[key] is not None:
            (p, newkey) = self._proxies[key]
            p.unpublish(newkey)
        else:
            self._publishers[key] = None

    def unsubscribe(self, key, subscriber):
        if self._proxies[key] is not None:
            (p, newkey) = self._proxies[key]
            p.unsubscribe(newkey, subscriber)
        else:
            self._subscribers[key].remove(subscriber)

    def proxy(self, key, p, newkey=None):
        if key not in self:
            self.__missing__(key)
        if newkey is None:
            newkey = key
        self._proxies[key] = (p, newkey)

    def unproxy(self, key):
        self._proxies[key] = None


# Test code
if __name__ == "__main__":
    import sys
    o = pubsub()

    # Non-existent key gets auto-created with None value
    print("Auto-created key 'foo' value:", o['foo'])

    # Add some subscribers
    # First is a bare function
    def print_len(x):
        print("len=%i" % (len(x), ))
    o.subscribe('foo', print_len)

    # The second is a class member function
    class subber(object):
        def __init__(self, param):
            self._param = param

        def printer(self, x):
            print(self._param, repr(x))
    s = subber('param')
    o.subscribe('foo', s.printer)

    # The third is a lambda function
    o.subscribe('foo', lambda x: sys.stdout.write('val=' + repr(x) + '\n'))

    # Update key 'foo', will notify subscribers
    print("Updating 'foo' with three subscribers:")
    o['foo'] = 'bar'

    # Remove first subscriber
    o.unsubscribe('foo', print_len)

    # Update now will only trigger second and third subscriber
    print("Updating 'foo' after removing a subscriber:")
    o['foo'] = 'bar2'

    # Publish a key as a function, in this case, a lambda function
    o.publish('baz', lambda: 42)
    print("Published value of 'baz':", o['baz'])

    # Unpublish the key
    o.unpublish('baz')

    # This will return None, as there is no publisher
    print("Value of 'baz' with no publisher:", o['baz'])

    # Set 'baz' key, it gets cached
    o['baz'] = 'bazzz'

    # Now will return cached value, since no provider
    print("Cached value of 'baz' after being set:", o['baz'])



#!/usr/bin/env python
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import math

from gnuradio import gr, blocks
from . import fec_python as fec


class capillary_threaded_encoder(gr.hier_block2):
    def __init__(self, encoder_list_0, input_size=gr.sizeof_char, output_size=gr.sizeof_char):
        gr.hier_block2.__init__(self, "Capillary Threaded Encoder",
                                gr.io_signature(1, 1, input_size),
                                gr.io_signature(1, 1, output_size))

        self.encoder_list_0 = encoder_list_0

        check = math.log10(len(self.encoder_list_0)) / math.log10(2.0)
        if(abs(check - int(check)) > 0.0):
            gr.log.info(
                "fec.capillary_threaded_encoder: number of encoders must be a power of 2.")
            raise AttributeError

        self.deinterleaves_0 = []
        for i in range(int(math.log(len(encoder_list_0), 2))):
            for j in range(int(math.pow(2, i))):
                self.deinterleaves_0.append(blocks.deinterleave(input_size,
                                                                fec.get_encoder_input_size(encoder_list_0[0])))

        self.generic_encoders_0 = []
        for i in range(len(encoder_list_0)):
            self.generic_encoders_0.append(fec.encoder(encoder_list_0[i],
                                                       input_size, output_size))

        self.interleaves_0 = []
        for i in range(int(math.log(len(encoder_list_0), 2))):
            for j in range(int(math.pow(2, i))):
                self.interleaves_0.append(blocks.interleave(output_size,
                                                            fec.get_encoder_output_size(encoder_list_0[0])))

        rootcount = 0
        branchcount = 1
        for i in range(int(math.log(len(encoder_list_0), 2)) - 1):
            for j in range(int(math.pow(2, i))):
                self.connect(
                    (self.deinterleaves_0[rootcount], 0), (self.deinterleaves_0[branchcount], 0))
                self.connect(
                    (self.deinterleaves_0[rootcount], 1), (self.deinterleaves_0[branchcount + 1], 0))
                rootcount += 1
                branchcount += 2

        codercount = 0
        for i in range(len(encoder_list_0) // 2):
            self.connect(
                (self.deinterleaves_0[rootcount], 0), (self.generic_encoders_0[codercount], 0))
            self.connect(
                (self.deinterleaves_0[rootcount], 1), (self.generic_encoders_0[codercount + 1], 0))
            rootcount += 1
            codercount += 2

        rootcount = 0
        branchcount = 1
        for i in range(int(math.log(len(encoder_list_0), 2)) - 1):
            for j in range(int(math.pow(2, i))):
                self.connect(
                    (self.interleaves_0[branchcount], 0), (self.interleaves_0[rootcount], 0))
                self.connect(
                    (self.interleaves_0[branchcount + 1], 0), (self.interleaves_0[rootcount], 1))
                rootcount += 1
                branchcount += 2

        codercount = 0
        for i in range(len(encoder_list_0) // 2):
            self.connect(
                (self.generic_encoders_0[codercount], 0), (self.interleaves_0[rootcount], 0))
            self.connect(
                (self.generic_encoders_0[codercount + 1], 0), (self.interleaves_0[rootcount], 1))
            rootcount += 1
            codercount += 2

        if((len(self.encoder_list_0)) > 1):
            self.connect((self, 0), (self.deinterleaves_0[0], 0))
            self.connect((self.interleaves_0[0], 0), (self, 0))
        else:
            self.connect((self, 0), (self.generic_encoders_0[0], 0))
            self.connect((self.generic_encoders_0[0], 0), (self, 0))

    def get_encoder_list_0(self):
        return self.encoder_list_0

    def set_encoder_list_0(self, encoder_list_0):
        self.encoder_list_0 = encoder_list_0



#!/usr/bin/env python
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


def bitreverse(mint):
    res = 0
    while mint != 0:
        res = res << 1
        res += mint & 1
        mint = mint >> 1
    return res


const_lut = [2]
specinvert_lut = [[0, 2, 1, 3]]


def bitflip(mint, bitflip_lut, index, csize):
    res = 0
    cnt = 0
    mask = (1 << const_lut[index]) - 1
    while (cnt < csize):
        res += (bitflip_lut[(mint >> cnt) & (mask)]) << cnt
        cnt += const_lut[index]
    return res


def read_bitlist(bitlist):
    res = 0
    for i in range(len(bitlist)):
        if int(bitlist[i]) == 1:
            res += 1 << (len(bitlist) - i - 1)
    return res


def read_big_bitlist(bitlist):
    ret = []
    for j in range(0, len(bitlist) / 64):
        res = 0
        for i in range(0, 64):
            if int(bitlist[j * 64 + i]) == 1:
                res += 1 << (64 - i - 1)
        ret.append(res)
    res = 0
    j = 0
    for i in range(len(bitlist) % 64):
        if int(bitlist[len(ret) * 64 + i]) == 1:
            res += 1 << (64 - j - 1)
        j += 1
    ret.append(res)
    return ret


def generate_symmetries(symlist):
    retlist = []
    if len(symlist) == 1:
        for i in range(len(symlist[0])):
            retlist.append(symlist[0][i:] + symlist[0][0:i])
        invlist = symlist[0]
        for i in range(1, len(symlist[0]) / 2):
            invlist[i] = symlist[0][i + len(symlist[0]) / 2]
            invlist[i + len(symlist[0]) / 2] = symlist[0][i]
        for i in range(len(symlist[0])):
            retlist.append(symlist[0][i:] + symlist[0][0:i])
        return retlist



#
# Copyright 2018 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
""" Module to convert XML bindings to YAML bindings """


import click

from ..core import get_xml_candidates, ModToolUpdate
from ..tools import SequenceCompleter
from .base import block_name, run, cli_input, ModToolException


def xml_blockname_complete(ctx, param, incomplete: str):
    return sorted(
        name for name in get_xml_candidates()
        if name.startswith(incomplete)
    )


@click.command('update', short_help=ModToolUpdate.description)
@click.option('--complete', is_flag=True, default=None,
              help="Convert all the XML bindings to YAML.")
@click.option('-I', '--include-blacklisted', is_flag=True, default=None,
              help="Include XML files with blacklisted names in the conversion process")
@click.argument("blockname", nargs=1, required=False, metavar="BLOCK_NAME", shell_complete=xml_blockname_complete)
def cli(**kwargs):
    """ Update the XML bindings to YAML bindings """
    kwargs['cli'] = True
    self = ModToolUpdate(**kwargs)
    click.secho("GNU Radio module name identified: " +
                self.info['modname'], fg='green')
    get_blockname(self)
    run(self)


def get_blockname(self):
    """ Returns the blockname for block to be updated """
    if self.info['complete']:
        return
    block_candidates = get_xml_candidates()
    if self.info['blockname'] is None:
        with SequenceCompleter(block_candidates):
            self.info['blockname'] = cli_input(
                'Which block do you wish to update? : ')
    if not self.info['blockname'] or self.info['blockname'].isspace():
        raise ModToolException('Block name not specified!')
    if self.info['blockname'] not in block_candidates:
        choices = [x for x in block_candidates if self.info['blockname'] in x]
        if len(choices) > 0:
            click.secho("Suggested alternatives: " + str(choices), fg='yellow')
        raise ModToolException("The XML bindings does not exists!")
